;
; BeebDefs.asm
;
; Defines for the BBC Micro BBC Master, and Acorn Electron
;
; 2008-10-02, P.Harvey-Smith.
;

;
; Character defs
;

Null		= &00
Bell		= &07
Tab		= &09
CR		= &0A
LF		= &0D
Space		= &20 

;
; Processor defs
;

StackBase	= &0100		; Stack is always at &0100-&01FF
StackPage	= &01

SWPage		= &80		; Start page of Sidways memory
OSRomPage	= &C0		; Start page of OS ROM

SWBase		= &8000		; SWR base
SWLangEnt	= SWBase	; Language entry
SWServEnt	= SWBase+3	; Service entry
SWRName		= &8009		; SWR name / copyright

;
; Vectors
;

BRKVect		= &FFFE		; BRK instruction vector
IRQVect		= BRKVect
ResetVect	= &FFFC		; Reset vector
NMIVect		= &FFFA		; NMI Vector

;
; Vars in low RAM
;

LangWork	= &00		; &00-&8F are allocated to the current language
UsrArea		= &70		; Basic reserves &70-&8F for the user
EconetArea	= &90		; &90-&9f reserved for econet
CLIPtr		= &F2		; Pointer to CLI text after *
RomSel		= &F4		; Ram copy of ROM select reg at &FF30
AddrPtr		= &F6		; Paged ram address pointer, used by servies &0D and &0E

BreakType	= &028D		; Last break type
SWRFlags	= &02A1		; Sideways memory flags 1 per bank

CasOutBuf	= &0900 	; Cassette output buffer
RS423OutBuf	= CasOutBuf	; Also the RS423 buffer
CasInBuf	= &0A00		; Cassette input buffer (formally CasBuf).
RS423InBuf	= CasInBuf	; Also the RS423 buffer

;
; I/O Locations.
;

Fred		= &FC00			; base of fred area
Jim			= &FD00			; base of Jim area
Sheila		= &FE00		    ; base of Sheila area

CRTC6845	= Sheila+&00	; 6845 registers
ACIA6850	= Sheila+&08	; 6850 registers
SerialULA	= Sheila+&10	; Serial ULA regs
VideoULA	= Sheila+&20	; Video ULA regs
ROMPage		= Sheila+&30	; Paged rom selector
SystemVIA	= Sheila+&40	; System VIA regs
UserVIA		= Sheila+&60	; User VIA regs
FloppyCTRL	= Sheila+&80	; Floppy disk controler
EconetADLC	= Sheila+&A0	; Econet controler
AToD		= Sheila+&C0	; uPD7002 ADC
TubeULA		= Sheila+&E0	; Tube ULA

;
; Electron specific I/O Locations.
;

ROMPageElk  = Sheila+&05    ; Electron rom page register

PageOutBasicElk	= &0C		; Write this to ROMPageElk to page out basic

FloppyCtrlElk	= Fred+&C0		; Floppy control 
FloppyCmdElk	= Fred+&C4		; Wd1770 command register
FloppyTrkElk	= Fred+&C5		; Wd1770 track register
FloppySecElk	= Fred+&C6		; Wd1770 sector register
FloppyDataElk	= Fred+&C7		; Wd1770 data register

;
; SWR size
;

SWRSizeBytes	= &4000	; Length of SWR in bytes
SWRSizePages	= &40	; Length of SWR in 256 byte pages

;
; Fixed Romoffsets, note some entries beyond the title string depend on the
; number of chars preceeding them.
;

SWRLangEntry	= &00	; Language entry point
SWRServiceEntry	= &03	; Service entry point
SWRRomTypeByte	= &06	; Rom type flags, see below
SWRCopyrightPtr	= &07	; Pointer to the copyright string
SWRBinVersionNo	= &08	; Binary version number, not checked by OS
SWRTitle		= &09	; Ascii Title string, followed by zero, followed by ascii version no + zero 

; ROM type byte bits.
; Bits 7..4 specify the type of rom

ServiceRom		= &80	; Service rom entry present
LanguageRom		= &40	; Language rom entry present
SecondProcReloc	= &20	; Second processor reloaction address
ElectronFirmKey	= &10	; Set if rom supports electorn firm keys

; Bits 3..0, specify the target CPU

CPU6502Basic	= &00	; 6502/12 Basic ROM
CPUTurb06502	= &01	; Turbo 6502
CPU6502			= &02	; 6502, non basic ROM
CPU68000		= &03	; Motorola 68000
CPUPDP11		= &07	; JGH's PDP11, by general convention.
CPUZ80			= &08	; Zilog Z80
CPU32016		= &09	; National Semi 32016
CPUReservedA	= &0A	; Reserved
CPU80186		= &0B	; Intel/AMD 80186
CPU80286		= &0C	; Intel/AMD 80286
CPUArm			= &0D	; Acorn Risc Machine 

;
; ROM service calls, passed in A
;

ROMNOP			= &00	; NO-OP, all roms should ignore this
ROMAbsWSClaim	= &01	; Claim absolute workspace
ROMPrivWSClaim	= &02	; Claim private workspace
ROMAutoBoot		= &03	; Auto boot at reset, for FS roms
ROMCmdNotKnow	= &04	; *command unrecognised
ROMIntNotKnown	= &05	; Unknow inturrupt
ROMUserBRK		= &06	; The user executed a BRK instruction
ROMOSBYTE		= &07	; User issued an unknown OSBYTE function 
ROMOSWORD		= &08	; User issued an unknown OSWORD function 
ROMHelpMe		= &09	; *HELP issued
ROMClaimStatic	= &0A	; Claim static workspace
ROMNMIRelease	= &0B	; The current user of NMI space no longer needs it
ROMNMIClaim		= &0C	; Claim the NMI space
ROMROMFSInit	= &0D	; Initialise the ROM FS
ROMROMFSBget	= &0E	; Get a byte from ROM FS
ROMVectorsClaim	= &0F	; Vectors claimed
ROMSpoolExClose	= &10	; *SPOOL or *EXEC file closing
ROMExImplodeFon	= &11	; Fonts are about to Explode/Implode
ROMSelectFS		= &12	; Select a fileing system
ROMTubePostInit	= &FE	; Tube post initialisation
ROMTubeMainInit	= &FF	; Tube main initialisation

;
; MOS calls, As documented in Advanced user guide
;

OSCLI		= &FFF7	; Command line interface, interprets * commands etc
OSBYTE		= &FFF4	; *FX/OSByte calls, 
OSWORD		= &FFF1	; OSWORD type calls
OSWRCH		= &FFEE	; Write a character to the currently selected stream
OSNEWL		= &FFE7	; Write a newline to the screen
OSASCI		= &FFE3	; Write ASCII character in A, &D->CR,LF
OSRDCH		= &FFE0	; Get a character from input stream
OSFILE		= &FFDD	; Load/Save file
OSARGS		= &FFDA	; Load/Save parameters
OSBGET		= &FFD7	; Get a byte from a file
OSBPUT		= &FFD4	; Put a byte in file
OSGBPB		= &FFD1	; Multi byte GET/PUT
OSFIND		= &FFCE	; Open/Close a file
NNWRCH		= &FFCB	; Non vectored wtite character
NVRDCH		= &FFC8	; Non vectored read character
GSREAD		= &FFC5	; Read a byte from a string
GSINIT		= &FFC2	; String input initialise
OSEVEN		= &FFBF	; Generate an event
OSRDSC		= &FFB9	; Read a byte from screen or paged ram/rom
OSWRSC		= &FFB3	; Write a byte to screen or paged ram/rom OS 2.x and above

;
; OSBYTE calls, passed in A
;

OSBPrintVer		= &00	; Print operating system version
OSBUser			= &01	; User OSBYTE call, read/write location &281
OSBSelInput		= &02	; Select input stream
OSBSelOutput	= &03	; Select output stream
OSBEditCursor	= &04 	; Enable/disable cursor editing
OSBSetPrinter	= &05	; Select printer destination
OSBPrinterIgnore= &06	; Set character ignored by printer
OSBSetRxBaud	= &07	; Set RS423 baud rate for receiving data
OSBSetTxBaud	= &08	; Set RS423 baud rate for data transmission
OSBSetFlashMark	= &09	; Set flashing colour mark state duration
OSBSetFlashSpace= &0A	; Set flashing colour space state duration
OSBSetKeyDelay	= &0B	; Set keyboard auto-repeat delay interval
OSBSetKeyRate	= &0C	; Set keyboard auto-repeat rate
OSBDisableEvent	= &0D	; Disable events
OSBEnableEvent	= &0E 	; Enable events
OSBFlushBufClass= &0F 	; Flush selected buffer class
OSBSelADCChan	= &10	; Select ADC channels to be sampled
OSBForceADC		= &11	; Force an ADC conversion
OSBResetKeys	= &12	; Reset soft keys
OSBWAitVsync	= &13	; Wait for vertical sync
OSBExplode		= &14	; Explode soft character RAM allocation
OSBFlushBuffer	= &15	; Flush specific buffer

;
; Calls &15-&74 not used by OS 1.2
;

OSBReadVDU		= &75	; Read VDU status
OSBUpdateLEDS	= &76	; Reflect keyboard status in LEDs
OSBCloseSpoolEx	= &77	; Close any SPOOL or EXEC files
OSBWriteCurKeys	= &78	; Write current keys pressed information
OSBScanKbd		= &79	; Perform keyboard scan
OSBScanKey16	= &7A	; Perform keyboard scan from 16 (&10)
OSBPrinterDorm	= &7B	; Inform OS, printer driver going dormant
OSBClearESC		= &7C	; Clear ESCAPE condition
OSBSetESC		= &7D	; Set ESCAPE condition
OSBAckESC		= &7E	; Acknowledge detection of ESCAPE condition
OSBCheckEOF		= &7F	; Check for EOF on an open file

OSBReadADC 		= &80 	; Read ADC channel or get buffer status
OSBReadKeyLimit = &81 	; Read key with time limit
OSBReadMighAddr = &82 	; Read machine high order address
OSBTopOSram 	= &83 	; Read top of OS RAM address (OSHWM)
OSBReadHIMEM 	= &84 	; Read bottom of display RAM address (HIMEM)
OSBreadBotMode 	= &85 	; Read bottom of display address for a given MODE
OSBReadCurPOS 	= &86 	; Read text cursor position (POS and VPOS)
OSBReadCharPOS 	= &87 	; Read character at cursor position
OSBDoCode 		= &88 	; Perform *CODE
OSBDoMotor 		= &89 	; Perform *MOTOR
OSBBufInsValue 	= &8A 	; Insert value into buffer
OSBDoOpt 		= &8B 	; Perform *OPT
OSBDoTape 		= &8C 	; Perform *TAPE
OSBDoRom 		= &8D 	; Perform *ROM
OSBEnterLangROM = &8E 	; Enter language ROM
OSBPagedROMReq 	= &8F 	; Issue paged ROM service request
OSBDoTV 		= &90 	; Perform *TV
OSBBufGetChar 	= &91 	; Get character from buffer
OSBReadFred 	= &92 	; Read from FRED, 1 MHz bus
OSBWriteFred 	= &93 	; Write to FRED, 1 MHz bus
OSBReadJim 		= &94 	; Read from JIM, 1 MHz bus
OSBWriteJim 	= &95 	; Write to JIM, 1 MHz bus
OSBReadSheila 	= &96 	; Read from SHEILA, mapped I/O
OSBWriteSheila 	= &97 	; Write to SHEILA, mapped I/O
OSBExamBufStat 	= &98 	; Examine buffer status
OSBBufInsChar 	= &99 	; Insert character into input buffer
OSBWriteULACtrl = &9A 	; Write to video ULA control register and copy
OSBWriteULAPal 	= &9B 	; Write to video ULA palette register and copy
OSBRW6850Ctrl 	= &9C 	; Read/write 6850 control register and copy
OSBTubeBput 	= &9D 	; Fast Tube BPUT
OSBReadSpeechP 	= &9E 	; Read from speech processor
OSBWriteSpeechP = &9F 	; Write to speech processor
OSBReadVDUVal 	= &A0 	; Read VDU variable value
OSBReadOSVarSL 	= &A6 	; Read start address of OS variables (low byte)
OSBReadOSVarSH 	= &A7 	; Read start address of OS variables (high byte)
OSBReadRomPtrL 	= &A8 	; Read address of ROM pointer table (low byte)
OSBReadRomPtrH 	= &A9 	; Read address of ROM pointer table (high byte)
OSBReadRomInfL 	= &AA 	; Read address of ROM information table (low byte)
OSBReadRomInfH 	= &AB 	; Read address of ROM information table (high byte)
OSBReadKeyTranL = &AC 	; Read address of key translation table (low byte)
OSBReadKeyTranH = &AD 	; Read address of key translation table (high byte)
OSBReadOSVDUVL 	= &AE 	; Read start address of OS VDU variables (low byte)
OSBReadOSVDUVH 	= &AF 	; Read start address of OS VDU variables (high byte)
OSBRWcfsTimeOut = &B0 	; Read/write CFS timeout counter
OSBRWInputSrc 	= &B1 	; Read/write input source
OSBRWKbdSem 	= &B2 	; Read/write keyboard semaphore
OSBRWPrimeOSHWM = &B3 	; Read/write primary OSHWM
OSBRWCurrOSHWM 	= &B4 	; Read/write current OSHWM
OSBRWrs423Mode 	= &B5 	; Read/write RS423 mode
OSBRWCharExploS = &B6 	; Read character definition explosion state
OSBRECasRomSw 	= &B7 	; Read/write cassette/ROM filing system switch
OSBReadRULACtrl = &B8 	; Read RAM copy of video ULA control register
OSBReadRULAPal 	= &B9 	; Read RAM copy of video ULA palette register
OSBRWRomBrk 	= &BA 	; Read/write ROM number active at last BRK (error)
OSBRWBasicSkt 	= &BB 	; Read/write number of ROM socket containing BASIC
OSBReadADCChan 	= &BC 	; Read current ADC channel
OSBRWMaxADCChan = &BD 	; Read/write maximum ADC channel number
OSBReadADCConvT = &BE 	; Read ADC conversion type
OSBRWrs432UseF 	= &BF 	; Read/write RS423 use flag
OSBRrs432CtrlF 	= &C0 	; Read RS423 control flag
OSBRWFlashCount = &C1 	; Read/write flash counter
OSBRWMarkPeriod = &C2 	; Read/write mark period count
OSBRWSpcPeriod 	= &C3 	; Read/write space period count
OSBRWKAutoDelay = &C4 	; Read/write keyboard auto-repeat delay
OSBRWKAutoPerd 	= &C5 	; Read/write keyboard auto-repeat period
OSBRWExecHand 	= &C6 	; Read/write *EXEC file handle
OSBRWSpoolHand 	= &C7 	; Read/write *SPOOL file handle
OSBRWEscBreakE 	= &C8 	; Read/write ESCAPE, BREAK effect
OSBRWEconetKDis = &C9 	; Read/write Econet keyboard disable
OSBRWkeyStatB 	= &CA 	; Read/write keyboard status byte
OSBRWrs432HandS = &CB 	; Read/write RS423 handshake extent
OSBRWrs432InpS 	= &CC 	; Read/write RS423 input suppression flag
OSBRWCasRS432F 	= &CD 	; Read/write cassette/RS423 selection flag
OSBRWEcoOSInt 	= &CE 	; Read/write Econet OS call interception status
OSBRWEcoOSRDCHI = &CF 	; Read/write Econet OSRDCH interception status
OSBRWEcoOSWRCHI = &D0 	; Read/write Econet OSWRCH interception status
OSBRWSpeechSupS = &D1 	; Read/write speech suppression status
OSBRWBellSupS 	= &D2 	; Read/write sound suppression status
OSBRWBellChan 	= &D3 	; Read/write BELL channel
OSBRWBellEnvo 	= &D4 	; Read/write BELL envelope number/amplitude
OSBRWBellFreq 	= &D5 	; Read/write BELL frequency
OSBRWBellLen 	= &D6 	; Read/write BELL duration
OSBRWStartMess 	= &D7 	; Read/write startup message and !BOOT options
OSBRWKeyStrLen 	= &D8 	; Read/write length of soft key string
OSBRWLinesPrint = &D9 	; Read/write number of lines printed since last page
OSBRWNumVDUQ 	= &DA 	; Read/write number of items in VDU queue
OSBRWTabCharVal = &DB 	; Read/write TAB character value
OSBRWEscCharVal = &DC 	; Read/write ESCAPE character value
OSBRWCharC0CFSt = &DD 	; Read/write character &CO to &CF status
OSBRWCharD0DFSt = &DE 	; Read/write character &DO to &DF status
OSBRWCharE0EFSt = &DF 	; Read/write character &EO to &EF status
OSBRWCharF0FFSt = &E0 	; Read/write character &FO to &FF status
OSBRWFnKeySt 	= &E1 	; Read/write function key status
OSBRWShFnKeySt  = &E2 	; Read/write SHIFT+ function key status
OSBRWCtFnKeySt 	= &E3 	; Read/write CTRL+function key status
OSBRWCtShFnKeySt= &E4 	; Read/write CTRL+SHIFT+function key status
OSBRWEscSt 		= &E5 	; Read/write ESCAPE key status
OSBRWEscEffects = &E6 	; Read/write flags determining ESCAPE effects
OSBRWUser65Mask = &E7 	; Read/write IRQ bit mask for user 6522
OSBRW6850Mask 	= &E8 	; Read/write IRQ bit mask for 6850
OSBRWSys65Mask 	= &E9 	; Read/write IRQ bit mask for system 6S22
OSBRWTubeFlag 	= &EA 	; Read flag indicating Tube presence
OSBRWSpeechFlag = &EB 	; Read flag indicating speech processor presence
OSBRWCharDestSt = &EC 	; Read/write write character destination status
OSBRWCursEditSt = &ED 	; Read/write cursor editing status
OSBRWLoc27E 	= &FE 	; Read/write location &27E, not used by 05 1.20
OSBRWLoc27F 	= &EF 	; Read/write location &27F, not used by 05 1.20
OSBRWLoc280 	= &F0 	; Read/write location &280, not used by 05 1.20
OSBRWLoc281 	= &F1 	; Read/write location &281, used by *FX 1
OSBRWRAMULACP 	= &F2 	; Read RAM copy of serial processor ULA
OSBRWTimerSwSt 	= &F3 	; Read/write timer switch state
OSBRWSoftkeyCon = &F4 	; Read/write soft key consistency flag
OSBRWPrintDestF = &F5 	; Read/write printer destination flag
OSBRWCharPrintI = &F6 	; Read/write character ignored by printer
OSBRWBrkCode1 	= &F7 	; Read/write first byte of BREAK intercept code
OSBRWBrkCode2 	= &F8 	; Read/write second byte of BREAK intercept code
OSBRWBrkCode3 	= &F9 	; Read/write third byte of BREAK intercept code
OSBRWLoc28A 	= &FA 	; Read/write location &28A, not used by OS1.20
OSBRWLoc28B 	= &FB 	; Read/write location &28B, not used by OS1.20
OSBRWLangROMNo 	= &FC 	; Read/write current language ROM number
OSBRWLastBrkTyp = &FD 	; Read/write last BREAK type
OSBRWAvailRAM 	= &FE 	; Read/write available RAM
OSBRWStartOpts 	= &FF 	; Read/write start up options

;
; OSWord calls, passed in A
;

OSWRdLineToMem	= &00	; Read line from currently selected input into memory.
OSWReadSysClk	= &01	; Read system clock.
OSWWriteSysClk	= &02	; Write system clock.
OSWReadIntTimer	= &03	; Read interval timer.
OSWWriteIntTimer= &04	; Write interval timer.
OSWReadIOPrByte	= &05	; Read byte of I/O processor memory.
OSWWriteIOPrByte= &06	; Write byte of I/O processor memory.
OSWSoundCmd		= &07	; Perform a SOUND command.
OSWDefEnvolope	= &08	; Define an ENVELOPE.
OSWReadPixelVal	= &09	; Read pixel value.
OSWReadCharDef	= &0A	; Read character definition.
OSWReadPal		= &0B	; Read palette value for a given logical colour.
OSWWritePal		= &0C	; Write palette value for a given logical colour.
OSWReadCursPos	= &0D	; Read previous and current graphics cursor positions.

;
; Buffer offsets for OSFILE
;
OSFBufferLen	= &12	; Total length of buffer

OSFFileNameAddr	= &00	; Address of filename
OSFLoadAddr		= &02	; Load address of file
OSFExecAddr		= &06	; Exec address of file
OSFStartAddr	= &0A	; Start address for save /
OSFLength		= &0A	; Length of file otherwise
OSFEndAddr		= &0E	; End address for save /
OSFAttributes	= &0E	; Attributes otherwise

;
; Operation codes for OSFILE, passed in A
;
OSFSaveBlock	= &00	; Save a block of memory to file
OSFWriteCat		= &01	; Write catalog info from buffer
OSFWriteLoad	= &02	; Write load address from buffer
OSFWriteExec	= &03	; Write the exec address from buffer
OSFWriteAttr	= &04	; Write attributes from buffer
OSFReadCat		= &05	; Read catalog info into buffer, filetype into A
OSFDeleteFile	= &06	; Delete the named file
OSFLoadFile		= &FF	; Load the named file

;
; OSFILE filetypes returned in A
;

OSFNothing		= &00	; Nothing found
OSFFileFound	= &01	; File found
OSFDirFound		= &02	; Dir found

;
; Buffer offsets for OSGBPB
;

OSGPBufferLen	= &0D	; Total length of buffer

OSGPHandle		= &00	; File handle, returned by OSFIND
OSGPDataPtr		= &01	; Pointer to data
OSGPLength		= &05	; Length of data to transfer
OSGPSequence	= &09	; Sequential pointer value to be used for transfer

;
; Operation codes for OSGBPB
;

OSGPPutNew		= &01	; Put bytes using new sequental ptr
OSGPPutIgnore	= &02	; Put bytes ignoring new sequental ptr
OSGPGetNew		= &03	; Get bytes using new sequental ptr
OSGPGetIgnore	= &04	; Get bytes ignoring new sequental ptr
OSGPGetTitle	= &05	; Get media title & start options
OSGPReadDir		= &06	; Read currently selected Dir
OSGPReadLib		= &07	; Read currently selected Lib 
OSGPReadFNames	= &08	; Read file names from dir

;
; Operation codes for OSFIND 
;
OSFDCloseFile	= &00	; Close file, Y=Handle
OSFDOpenRead	= &40	; Open a file for read Y:X=Addr of name
OSFDOpenWrite	= &80	; Open a file for write Y:X=Addr of name
OSFDOpenUpdate	= &C0	; Open a file for read & write Y:X=Addr of name

;
; Return codes for last reset type
; 

ResetSoft		= &00	; Soft reset, just break
ResetPowerOn	= &01	; Power on break
ResetHard		= &02	; CTRL-Break