;*************************************************************************
; *SRCMP <idsource> <iddest> (<start addr>) (<end addr>)                                         
;
; Compares the contents of Sideways bank idsource to iddest.
; Prints out the first hex address that differs, or $C000 if 
; they are identical.
;*************************************************************************

.DoCmpParamError	
	jmp	ErrMsgParamError
	
.DoCmpSramAddrError
	jmp	ErrMsgSramAddrError

.DoSRCMP

	jsr	GetRomIDS	; Get rom ids
	bcs	DoCmpParamError	; Error : exit
	
	jsr	GetHexWord	; Get start address if any
	bcs	DoSRCMPDefStart	; no start address use default
	
	sta	StartAddr+1	; Save start address
	stx	StartAddr
	
	jsr	NextNonSpace	; skip past spaces
	
	jsr	GetHexWord	; Get end address if any
	bcs	DoSRCMPDefEnd	; no start address use default
	
	jsr	CalcSWREndLen	; Calc end address
	bcs	DoCmpSramAddrError	; Address error : exit

	jmp	DoSRCMPCopy	; Parameters ok, go do compare
	
.DoSRCMPDefStart
	lda	#SWPage		; Set default start address
	sta	MemPtr+1
	lda	#0
	sta	MemPtr
.DoSRCMPDefEnd
	lda	#OSRomPage	; Set default end address
	sta	EndAddr+1
	lda	#0
	sta	EndAddr
	
;
; Copy the RAM comparing code onto the stack !
;
	
.DoSRCMPCopy
{
	ldy	#SRCmpRamEnd-SRCmpRamCode	; Get length to copy
.SRStackCopyLoop
	lda	SRCmpRamCode,Y		; Get a byte
	pha				; Put on stack
	dey				; Decrement pointer
	bmi	SRStackCopyEnd
	jmp	SRStackCopyLoop		; If more do them
.SRStackCopyEnd
}

	tsx				; Get exec address	
	lda	#&01			; Stack is in page &01
	pha
	
	txa				; Now stack SP
	pha				

	rts				; call stacked code
	
;
; Stacked code will jump to here on exit
;

.SRCmpEndRamExec
	tsx				; clean up stack
	txa
	sec
	adc 	#SRCmpRamEnd-SRCmpRamCode
	tax
	txs
	
	lda	MemPtr+1		; Get MSB
	jsr	OutHexByte		; Output it
	lda	MemPtr			; Get LSB
	jsr	OutHexByte		; Output it
	jsr	OutEOL			; And an EOL sequence

	jmp	CommandEndHandled

;
; This code is copied to RAM (on the stack), as it has to execute 
; from outside of the banked rom, as it will change banks !
;

.SRCmpRamCode
	ldy	MemPtr		; get LSB of start address
	ldx	StackFrame	; Point x to our stack frame
	lda	#0
	sta	MemPtr		; Zero LSB of start address
.SRCmpRamLoop	

if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	SourceID	; Get ID to process
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	SourceID	; Select source page
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif	
	lda	(MemPtr),Y	; Get byte from source
	pha			; save on stack

if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	DestID	        ; Get ID to process
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	DestID		; Select dest page
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif	
	pla			; Recover byte to compare
	
	cmp	(MemPtr),Y	; Compare to byte in dest page
	bne	SRCmpRamExit	; Not the same : exit
	
; Check to see if we have reached the end of the area being checked.
	
	lda	MemPtr+1	; Get page currently being scanned
	cmp	EndAddr+1,x	; Reached same page as end yet ?
	bne	SRIncNext	; nope, keep going
	
	tya			; Page the same, so check the offset
	cmp	EndAddr	,x
	beq	SRCmpRamExit	; Yep we're done.
	
.SRIncNext	
	iny			; Increment mem pointer low byte
	bne	SRCmpRamLoop	; Continue if not Zero
	
	inc	MemPtr+1	; Increment high byte
	lda	MemPtr+1	; Get memptr high byte
	cmp	#OSRomPage	; End of SRAM area ?
	bne	SRCmpRamLoop	; nope : do more pages
	
.SRCmpRamExit

if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	OurID	        ; Retieve our ROM id
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	OurID		; Retieve our ROM id
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif

	jmp	SRCmpEndRamExec	; Return to caller
.SRCmpRamEnd
