;*************************************************************************
; *SRCOPY <idsource> <iddest>                                          
;
; Copies the contents of Sideways bank idsource to iddest.
;*************************************************************************

.DoCopyParamError
	jmp	ErrMsgParamError

.DoSRCOPY
	jsr	GetRomIDS		; Get rom ids
	bcs	DoCopyParamError	; Error : exit
;
; Copy the RAM copying code onto the stack !
;
	
	ldy	#SRCopyRamEnd-SRCopyRamCode	; Get length to copy
.SRCopyStackCopyLoop
	lda	SRCopyRamCode,Y		; Get a byte
	pha				; Put on stack
	dey				; Decrement pointer
	bmi	SRCopyCmpCallOnStack
	jmp	SRCopyStackCopyLoop	; If more do them
	
.SRCopyCmpCallOnStack
	lda	#&00			; Save start address for copy
	sta	MemPtr
	lda	#&80
	sta	MemPtr+1

	tsx				; Get exec address	
	lda	#&01			; Stack is in page &01
	pha
	
	txa				; Now stack SP
	pha				

	rts				; call stacked code
	
.SRCopyEndRamExec
	tsx				; clean up stack
	txa
	sec
	adc 	#SRCopyRamEnd-SRCopyRamCode
	tax
	txs
	
.SRCopyDone
	jmp	CommandEndHandled

;
; This code is copied to RAM (on the stack), as it has to execute 
; from outside of the banked rom, as it will change banks !
;
	
.SRCopyRamCode
	ldy	#&00
.SRCopyLoop	

if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	SourceID	; Get ID to process
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	SourceID	; Select source page
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif	
	
	lda	(MemPtr),Y	; Get byte from source
	tax			; save in x

if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	DestID	        ; Get ID to process
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	DestID		; Select dest page
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif	
	
	txa			; Recover byte to transfer
	
	sta	(MemPtr),Y	; Put it in dest page
	iny			; Increment mem pointer low byte
	bne	SRCopyLoop	; Continue if not Zero
	
	inc	MemPtr+1	; Increment high byte
	lda	MemPtr+1	; Get memptr high byte
	cmp	#OSRomPage	; End of SRAM area ?
	bne	SRCopyLoop	; nope : do more pages
	
if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	OurID	        ; Retieve our ROM id
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	OurID		; Retieve our ROM id
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif

	jmp	SRCopyEndRamExec	; Return to caller
.SRCopyRamEnd
