;*************************************************************************
; *SRLOAD <FileName> <RomAddr> <id>  (<Q|S>) (<I>)                                        
;
; Loads a file into sideways ram id at specified address.
;
; Q=Quick, use OSHWM to (HIMEM-1) as a buffer
; S=Screenmem, use HIMEM to &7FFF as a buffer
; I=Init, put rom's status byte in mos table, saves having to reset
; 
; If neither Q or S is specified, then the &200 byte cassette buffer is used.
;*************************************************************************
;
; Intemediate jumps, so they can be in range of Bxx instructions.
;
	
.DoLoadSramAddrError
	jmp	ErrMsgSramAddrError

.DoLoadBadSRId
	jmp	ErrMsgBadSRId

.DoLoadParamError
	jmp	ErrMsgParamError

.DoSRLoad
	jsr	NextNonSpace		; Skip past spaces
	
	ldx	StackFrame		; Point to stack frame
	tya
	sta	SaveCliPtr,x		; Save ptr to name for later use
	
	jsr	NextSpace		; skip past filename
	bcs	DoLoadParamError	; Error : exit
	
	jsr	TerminateFilename	; Terminate filename for OSFILE
	jsr	NextNonSpace		; Skip past spaces

; Get load address into A:X
	jsr	GetHexWord		; Get hex byte
	bcs	DoLoadSramAddrError	; Error : exit

; Check load address is in the $8000-$BFFF range
	
	cmp	#SWPage		
	bcc	DoLoadSramAddrError	; Error : exit, to low
	cmp	#OSRomPage
	bcs	DoLoadSramAddrError	; Error : exit
	
	sta	MemPtr+1		; Save in memory pointer
	stx	MemPtr

	jsr	NextNonSpace		; Skip past spaces
	
; Get rom id
	jsr	GetDestID		; Get from command line
	bcs	DoLoadBadSRId		; Error : exit
		
; If we make it here we have a valid set of required params.
; check for optional flags
.DoLoadNextFlag
	jsr	GetFlag			; Get next flag
	bcs	DoLoadFlagsDone
	
	jsr	DoDecodeFlag		; Decode flags
	jmp	DoLoadNextFlag		; Check for more
	

.DoLoadFlagsDone

	jsr	DoSetupFileBuf		; Setup fileBuf depending on flags

;
; As OSGBPB has much lower performance than OSFILE for reading files
; we will check to see if we can read in the whole file in one go, and
; use OSFILE to do this if we can. This requires that we get the size of 
; the file to be loaded, and if it is smaller than or equal to the FileBuf
; then we will use OSFILE to load it in one go, else we will fall back to
; using the slower OSGBPB. 
;

; use OSFILE to get the size of the file to be loaded.

	clc
	ldx	StackFrame		; Point to stack frame
;	lda	CLIPtr			; Get low byte of CLI pointer
;	clc				; Make sure carry flag zero
;	adc	SaveCliPtr,x		; Add saved filename pointer
	lda	SaveCliPtr,x		; Get saved filename pointer
	sta	SFOSFFNameAddr,x
	lda	CLIPtr+1		; Get CLI page
	sta	SFOSFFNameAddr+1,x	; Save in control block
	
	ldy	#StackPage		; Y:X-> address of OSFBuf
	ldx	CtrlBufAddr
	lda	#OSFReadCat		; Read files catalog info to buffer
	jsr	OSFILE

	cmp	#OSFFileFound		; Did we find a file ?
	bne	SRLoadNotFound

	ldx	StackFrame		; Point to stack frame
	lda	SFOSFLength+3,x		; Check top 2 bytes of length, must be zero
	bne	SRLoadTooBig		
	lda	SFOSFLength+2,x		
	bne	SRLoadTooBig	
	lda	SFOSFLength+1,x		; now check that byte 1 <= &40
	cmp	#SWRSizePages+1		; error if bigger
	bcs	SRLoadTooBig	
	
	cmp	DTAPages,x		; See if file has less pages than buffer ?
	bcs	DoLoadOSGBPB		; yes : load by OSGBPB
	
; If we reach here then we can use OSFILE to load the file in, to do this we need to
; set the correct load address and then tell OSFILE to use our load address by setting
; byte 0 of the exec address to zero.

	lda	#&FF			; Load into IO processor
	sta	SFOSFLoadAddr+2,x
	sta	SFOSFLoadAddr+3,x
	
	lda	DTABasePage,x		; Get DTA page
	sta	SFOSFLoadAddr+1,x
	lda	#0			; on a page boundry so zero offset
	sta	SFOSFLoadAddr,x
	
	sta	SFOSFExecAddr,x		; Zero exec address byte
	
	lda	#OSFLoadFile		; Load file
	ldy	#StackPage		; Y:X-> address of OSFBuf
	ldx	CtrlBufAddr
	jsr	OSFILE
	
	cmp	#OSFFileFound		; Did we find a file ?
	bne	SRLoadNotFound

	lda	#EOFNoClose		; Push EOF file onto stack for later
	pha

; If we get here then the file has beeen loaded into memory we need to jump to the
; SWR copy loop	we also need to calculate EndAddr from DTABasePage+LengthLoaded

	ldx	StackFrame		; Point to stack frame
	
	lda	#0			; initialise EndAddr with DTABasePage:0
	sta	EndAddr,x
	lda	DTABasePage,x
	sta	EndAddr+1,x
	
	clc
	lda	SFOSFLength,x		; get LSB of loaded length
	adc	EndAddr,x		; Add to EndAddr
	sta	EndAddr,x		; save it back
	lda	SFOSFLength+1,x		; get MSB of loaded length
	adc	EndAddr+1,x
	sta	EndAddr+1,x		; save it back
	
	jmp	SRLoadDoStack		; Stack the copy code, and copy data
	

; Intemediate jumps, so they can be in range of Bxx instructions.

.SRLoadNotFound
	jmp	ErrMsgFileNotFound

.SRLoadTooBig
	jmp	ErrMsgFileTooBig

	

; If we get here then we need to use the slower OSGBPB to read the file in
; chunks.

.DoLoadOSGBPB
	clc
	ldx	StackFrame		; Point to stack frame
;	lda	CLIPtr			; Get low byte of CLI pointer
;	adc	SaveCliPtr,x		; Add saved filename pointer
	lda	SaveCliPtr,x		; Get saved filename pointer
	tax				; move it to X
	ldy	CLIPtr+1		; Get high byte
	
	lda	#OSFDOpenRead	        ; Open the file for read
	jsr	OSFIND			; Try opening the file
	
	cmp	#&00			; Check for valid handle 
	beq	SRLoadNotFound	        ; Not found, error
	
;
; Fill in OSGBPB parameter block
;
	ldx	StackFrame		; Point to stack frame
	sta	GPHandle,x		; save in buffer
	
.SRLoadFillParam
	ldx	StackFrame		; Point to stack frame

	lda	#0			; Offset always 0 as on page boundry
	sta	GPDataPtr,x
	
	lda	DTABasePage,x	        ; Base page of DTA
	sta	GPDataPtr+1,x
	
	lda	#&FF
	sta	GPDataPtr+2,x
	sta	GPDataPtr+3,x
	
	lda	#0			; Buffer size LSB always 0 as exact no of pages
	sta	GPLength,x
	
	lda	DTAPages,x		; Get no of pages
	sta	GPLength+1,x
	lda	#0			; Top 2 bytes 0
	sta	GPLength+2,x
	sta	GPLength+3,x
	
;
; Read a block from file at current file pointer
;
	lda	#OSGPGetIgnore		; Read bytes, ignore pointer from block
	ldy	#StackPage		; Y:X->GBPBbuffer address
	ldx	CtrlBufAddr
	jsr	OSGBPB	

.fileread
	
	bcc	RSRCalcBytes		; no error, continue reading
	
	lda	#EOFClose		; Push EOF flag onto stack
	pha
	
	jmp	RSRCalcBytesB	
.RSRCalcBytes

	lda	#NotEOF			; Push not EOF onto stack
	pha

;
; Work out if any bytes where actually read
;
; Since FileBuf is on a page boundry, if the LSB is not zero then we transfered at
; least that many bytes so may proceed. If the LSB is zero, then we also need to check
; that the MSB of the bytecount is non-zero, by subtracting the page no of the base of the 
; buffer from the page no of next byte to process.
;

.RSRCalcBytesB
	ldx	StackFrame		; Stack frame
	lda	GPDataPtr,x	
	bne	SRLoadDoStackB		; LSB not zero, so transfer
	
	lda	GPDataPtr+1,x	
	sec
	sbc	DTABasePage,x			
	bne	SRLoadDoStackB		; MSB not zero, so transfer

; If we get to here then no bytes where read, so exit and return to os.
	
	jmp	SRLoadCloseFile		; yep : EOF, exit

.SRLoadDoStackB
	lda	GPDataPtr,x		; Transfer last byte read to EndAddr
	sta	EndAddr,x
	lda	GPDataPtr+1,x		
	sta	EndAddr+1,x
	
;
; Push the copy code onto the stack, and execute it from there
;
	
.SRLoadDoStack
	ldy	#SRLoadRAMCopyEnd-SRLoadRAMCopy
.SRLoadDoStackLoop
	lda	SRLoadRAMCopy,y		; Get a byte
	pha				; save on stack
	dey				; decrement count
	bmi	RSRCallOnStack		; Done all ? yes, exit
	jmp	SRLoadDoStackLoop

.RSRCallOnStack
	lda	#0			; Fill in FileBufPtr LSB, alwsys 0 as on page 
	sta	FileBufPtr
	lda	DTABasePage,x		; Fill in FileBufPtr MSB
	sta	FileBufPtr+1
	
	tsx				; Get address to call
	lda	#StackPage
	pha
	txa
	pha
	rts				; call code on stack
;
; Increment memory pointer
;

.SRLoadCopyEnd	
	tsx				; clean up stack
	txa
	sec
	adc 	#SRLoadRAMCopyEnd-SRLoadRAMCopy
	tax
	txs

	pla				; Get EOF flag
	cmp	#EOFClose		; Check it
	beq	SRLoadCloseFile		; EOF Exit		

	cmp	#EOFNoClose		; Eof no close (loaded file using OSFILE)
	beq	SRLoadCheckInit		; Skip past close code

	lda	MemPtr+1		; Check for end of SW mem
	cmp	#OSRomPage		; Beyond end of sw ram
	bcs	SRLoadCloseFile		; Yep exit
	
.SRLoadPtrDone
	jmp 	SRLoadFillParam		; Do next block
	
.SRLoadCloseFile
	ldx	StackFrame		; Get file handle
	lda	GPHandle,x
	tay
	lda	#OSFDCloseFile		; Close the file
	jsr	OSFIND

	
.SRLoadCheckInit
; Check to see if the init flag was given, if so put the rom's type
; byte into the MOS table.

	lda	Flags,x			; Get flags
	and	#FlagInitRom		; Select flag we want
	beq	DoLoadDone		; no init return to MOS
	
	lda	RomTypeByte,x		; Get type byte
	pha	
	
	jsr	GetRomTypeTable		; Get table pointer into x:y
	
	sty	MemPtr+1		; Save in pointer
	stx	MemPtr
	
	ldy	DestID			; get destination rom id
	pla				; restore saved romtype
	
	sta	(MemPtr),y		; update table
	
.DoLoadDone
	jmp	CommandEndHandled
	
;
; Code copied into RAM on the machine stack to copy the loaded bytes
; from the file buffer to the sideways ram bank.
;
	
.SRLoadRAMCopy
	ldx	StackFrame		; Get pointer to StackFrame

if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	DestID	        ; Get ID to process
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	DestID		; Select dest page
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif	
	
	ldy	#0			; Zero byte count

.SRLoadRAMCopyLoop
	lda	(FileBufPtr),Y		; get a byte
	sta	(MemPtr),Y		; put it in bank
	iny				; increment pointer
	cpy	#0			; done a whole page ?
	bne	SRLoadRAMIncSrc		; nope increment and check source ptr 
	
	inc	FileBufPtr+1		; Move to next source page
	inc	MemPtr+1		; Move to next dest page
.SRLoadRAMIncSrc
	
;
; Check for end of buffer, first compare MSB, if they are different then we
; have not yet done all bytes, if they are the same, then, test LSB.
;
.SRLoadRAMCheckSrc
	lda	FileBufPtr+1		; Get MSB of source ptr
	cmp	EndAddr+1,x		; Compare to last byte read
	bne	SRLoadRAMCopyLoop	; not same, keep going loop again

; Now compare LSB, if LSB is the same then we are done.	

        tya                             ; Get LSB of source ptr
	cmp	EndAddr,x		; Compare to last byte read
	bne	SRLoadRAMCopyLoop	; not same, keep going loop again

	sty	MemPtr			; Save new MemPtr offset

	lda	SWBase+SWRRomTypeByte	; get rom type byte
	sta	RomTypeByte,x		; save it in stack frame

if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	OurID	        ; Retieve our ROM id
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	OurID		; Retieve our ROM id
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif

	jmp	SRLoadCopyEnd
.SRLoadRAMCopyEnd


;
; Flag decoding code, used by both SRLOAD and SRSAVE.
; on entry A contains the character of the flag from the command line
; on exit the flags byte on the stack is updated, and carry is clear
; or the flag byte is un-touched and carry is set (error)
;

.DoDecodeFlag
	ldx	StackFrame		; Point to stack frame
	cmp	#CharFlagQuick		; Check for flag char
	bne	DoDecodeNext1		; nope skip
	lda	#FlagQuick		; Get flag bit
	jmp	DoDecodeFlagOk		; Set it and exit

.DoDecodeNext1
	cmp	#CharFlagScreen		; Check for flag char
	bne	DoDecodeNext2		; nope skip
	lda	#FlagScreen		; Get flag bit
	jmp	DoDecodeFlagOk		; Set it and exit

.DoDecodeNext2
	cmp	#CharFlagInitRom	; Check for flag char
	bne	DoDecodeNext3		; nope skip
	lda	#FlagInitRom		; Get flag bit
	jmp	DoDecodeFlagOk		; Set it and exit
	
.DoDecodeNext3
	cmp	#CharFlagVerbose	; Check for flag char
	bne	DoDecodeNext4		; nope skip
	lda	#FlagVerbose		; Get flag bit
	jmp	DoDecodeFlagOk		; Set it and exit

.DoDecodeNext4

.DoDecodeFlagBad
	sec				; Flag error
	rts
	
.DoDecodeFlagOk
	ora	Flags,x			; Combine it
	sta	Flags,x			; Save it back

	clc				; Flag no error
	rts
	
;
; Setup FileBuf size and base params, defaults to &200 bytes
; in the cassette buffer, flags Q and S change this.
;
; Used by both SRLOAD and SRSAVE.
;

.DoSetupFileBuf
	ldx	StackFrame		; Get stack frame
	lda	#DefFileBufAddrMSB	; Default, cassette buffer
	sta	DTABasePage,x
	
	lda	#HI(DefFileBufSize)	; Default &0200 bytes
	sta	DTAPages,x
	
	lda	Flags,x			; Get flags
	and	#FlagQuick+FlagScreen	; mask out all but the ones we want
	beq	DoSetupFileBufExit	; zero, no flags set so use default
	
	cmp	#FlagQuick		; Check for quick flag
	bne	DoSetupFileBufScreen	; Zero, quick flag not set so must be screen

	lda	#OSBTopOSram		; Get OSHWM into y:x
	jsr	OSBYTE

	txa				; get low byte
	beq	DoSetupFileBuf1		; zero, so leave page alone
	iny				; bump y to next full page

.DoSetupFileBuf1
	tya				; save hi byte
	ldx	StackFrame		; Get stack frame
	sta	DTABasePage,x		; save start page

	lda	#OSBReadHIMEM		; Get bottom of screen ram in y:x
	jsr	OSBYTE

	txa				; get low byte
	beq	DoSetupFileBuf2		; zero, so leave page alone
	dey				; bump y to previous full page

.DoSetupFileBuf2
	tya				; Move hi byte to a
	ldx	StackFrame		; Get stack frame
	
	sec
	sbc	DTABasePage,x		; Work out length in pages

	jmp	DoSetupFileBufSize	; Check suize and exit
.DoSetupFileBufScreen
	lda	#OSBReadHIMEM		; Get bottom of screen ram in y:x
	jsr	OSBYTE

	txa				; get low byte
	beq	DoSetupFileBuf3		; zero, so leave page alone
	iny				; bump y to next full page

.DoSetupFileBuf3

	tya				; Move hi byte to a
	ldx	StackFrame		; Get stack frame
	sta	DTABasePage,x		; save start page
	
	sec
	lda	#&80			; top of ram
	sbc	DTABasePage,x		; Work out length in pages
	
.DoSetupFileBufSize
	cmp	#SWRSizePages		; Greater than &40 pages ?
	bcc	DoSetupFileBufSetPages	; no leave alone
	
	lda	#SWRSizePages		; Set pages to same size as max swram
	
.DoSetupFileBufSetPages
	sta	DTAPages,x
	
.DoSetupFileBufExit	
	rts