;*************************************************************************
; *SRREAD <source start> <source end> <sram start> <id>
; *SRREAD <source start> <+Length> <sram start> <id>                                     
;
; Copies data from normal ram to sideways ram
;
; *SRWRITE <source start> <source end> <sram start> <id>
; *SRWRITE <source start> <+Length> <sram start> <id>                                     
;
;*************************************************************************
;
; Intemediate jumps, so they can be in range of Bxx instructions.

.DoSRRead
	jsr	SRRWGetParams		; Get params for command line
	
; swap source and sest buffer pointers, this way we can use common code
; for both read and write.

	lda	SourceBufPtr		; pickup source and stack it
	pha
	lda	SourceBufPtr+1		
	pha
	
	lda	DestBufPtr		; Copy dest to source
	sta	SourceBufPtr
	lda	DestBufPtr+1	
	sta	SourceBufPtr+1
	
	pla				; Now copy old Source to dest
	sta	DestBufPtr+1
	pla	
	sta	DestBufPtr
	
	jmp	SRRWDoStack		; Go copy the data
	
.DoSRWrite
	jsr	SRRWGetParams		; Get params for command line
	jmp	SRRWDoStack		; Go copy the data

;
; Intemediate jumps, so they can be in range of Bxx instructions.
;

.SRRWParamError
	jmp	ErrMsgParamError
	
.SRRWSramAddrError
	jmp	ErrMsgSramAddrError

.SRRWBadSRId
	jmp	ErrMsgBadSRId

.SRRWNotFound
	jmp	ErrMsgFileNotFound


;
; Get parameters from the command line.
;

.SRRWGetParams
	jsr	NextNonSpace		; Skip past spaces

; Get source address into A:X
	jsr	GetHexWord		; Get hex word
	bcs	SRRWSramAddrError	; Error : exit

	cmp	#SWPage			; Must be below SWR
	bcs	SRRWSramAddrError	; Not, flag error
	
	sta	SourceBufPtr+1		; Save source pointer
	stx	SourceBufPtr
	
	jsr	NextNonSpace		; Skip past spaces

; Get source length into A:X
	jsr	GetHexWord		; Get hex word
	bcs	SRRWSramAddrError	; Error : exit
	
	jsr	GetOffsetInCarry	; Get offset/absolute flag
	bcs	SWRWDoOffset
	
	jsr	CalcLengthFromEnd	; Absolute, so work out length
	jmp	SRRWValidateSource
	
.SWRWDoOffset
	jsr	CalcEndFromLen  	; Calculate end address given length
	
.SRRWValidateSource
	lda	EndAddr+1,x		; Get end address
	cmp	#SWPage			; Past end of normal ram ?
	bcs	SRRWSramAddrError	; Not, flag error
	
	jsr	NextNonSpace		; Skip past spaces

; Get dest address into A:X
	jsr	GetHexWord		; Get hex word
	bcs	SRRWSramAddrError	; Error : exit

	sta	DestBufPtr+1		; Save destination address
	stx	DestBufPtr
	
	cmp	#SWPage		
	bcc	SRRWSramAddrError	; Error : exit, to low
	cmp	#OSRomPage
	bcs	SRRWSramAddrError	; Error : exit
	
	jsr	NextNonSpace		; Skip past spaces
	jsr	GetID			; Get rom ID
	bcs	SRRWBadSRId		; invalid ID : exit
	sta	DestID			; Save ROM ID
	
	rts
	
;
; Push the copy code onto the stack, and execute it from there
;
	
.SRRWDoStack
	ldy	#SRRWRAMCopyEnd-SRRWRAMCopy
.SRRWDoStackLoop
	lda	SRRWRAMCopy,y		; Get a byte
	pha				; save on stack
	dey				; decrement count
	bmi	SRRWCallOnStack		; Done all ? yes, exit
	jmp	SRRWDoStackLoop

.SRRWCallOnStack
	tsx				; Get address to call
	lda	#StackPage
	pha
	txa
	pha
	rts				; call code on stack

.SRRWStackDone
	tsx				; clean up stack
	txa
	sec
	adc 	#SRRWRAMCopyEnd-SRRWRAMCopy
	tax
	txs
	
	jmp	CommandEndHandled	; Ok we're done


;
; Code copied into ram on stack to do copy
;

.SRRWRAMCopy

if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	DestID	        ; Get ID to process
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	DestID		; Select dest page
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif	

	ldx	StackFrame		; Point x at stack frame
	ldy	#0			; Zero byte count
.SRRWRAMCopyLoop

	lda	(SourceBufPtr),y	; get a byte from source 
	sta	(DestBufPtr),y		; put it in dest
	iny				; increment source pointer
	cpy	#0			; done a whole page ?
	bne	SRRWRAMCheckCount	; nope check byte count

	inc	SourceBufPtr+1		; Move to next source page
	inc	DestBufPtr+1		; Move to next dest page
.SRRWRAMCheckCount
	dec	DataLen,x		; Dectement data count LSB
	lda	DataLen,x		; get LSB of count
	cmp	#&FF			; wrapped ?
	bne	SRRWRAMCopyLoop		; nope do next
	
	dec	DataLen+1,x		; Dectement data count MSB
	lda	DataLen+1,x		; get MSB of count
	cmp	#&FF			; wrapped ?
	bne	SRRWRAMCopyLoop		; nope do next

; If we reach here then we have copyed all bytes, so return to rom	

if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	OurID	        ; Retieve our ROM id
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	OurID		; Retieve our ROM id
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif

	jmp	SRRWStackDone		; Jump back to rom
.SRRWRAMCopyEnd