;*************************************************************************
; *ROMS (<id>) (<id>)
;
; List all recognised sideways rom/ram banks.
;*************************************************************************

.DoRomsParmError
	jmp	ErrMsgParamError
	
.DoRoms	
	jsr	Get1or2RomIDS		; Get rom IDS
	bcs	DoRomsAll		; on error list all
	jsr	SwapIDS			; Swap rom IDs if needed
	jmp	DoRomFlags		; Roms set go display them
	
.DoRomsAll
	lda	#&0F
	sta	SourceID		; Start at rom #F
.DoRomsDest
	lda	#&00			; Last rom $00-$01=$FF
	sta	DestID

.DoRomFlags
        lda     #0
        sta     ListFlags               ; Zero flags
        jsr     GetFlag2                ; Don't skip non space as we might already be
                                        ; pointing to flag.
        jmp     RomsNextFlag1           ; continue                          
.RomsNextFlag     
        jsr     GetFlag                 ; get a flag if present
.RomsNextFlag1     
        bcs	DoRomsSet               ; no flags skip on
        
        cmp     #CharFlagVerbose        ; Verbose flag ?
        bne     RomsNxetFlag1
        lda     #FlagVerbose            ; Set flag
        jmp     RomsFlagDone
        
.RomsNxetFlag1

        jmp     RomsNextFlag            ; process next flag

.RomsFlagDone
        ora     ListFlags               ; combine flags
        sta     ListFlags               ; save them back
        
        jmp     RomsNextFlag            ; process next flag


.DoRomsSet  
        clc
	jsr	GetRomTypeTable		; Get pointer to rom type table in Y:X
	
	stx	TablePtr		; Save in TablePtr
	sty	TablePtr+1

.DoRomsLoop
	ldy	SourceID		; recover RomID
	lda	(TablePtr),Y		; Get current rom's byte
	sta	InfoByte		; Save rom info byte

        lda     ListFlags               ; get flags
        and     #FlagVerbose            ; Verbose set ?
        bne     RomIsValid              ; yes, don't check valid flag

        lda     InfoByte                ; recover Infobyte
	beq	DoRomsNext		; Not a valid rom, skip on
	
.RomIsValid
	ldx	#RomNoMess-TextMessBase	; print message
	jsr	DisplayMess
	
	lda	SourceID		; Get rom number
	jsr	OutHexDigit		; output it

	
	ldx	#RomParamMess-TextMessBase	; print message
	jsr	DisplayMess
	
	lda	InfoByte		; Get saved info byte
	and	#ServiceRom		; Test service rom flag
	beq	DoRomsNotSvc
	
	lda	#'S'			; Show that this is a service
	jmp	DoRomsLang
	
.DoRomsNotSvc
	lda	#Space			; show that it is not

.DoRomsLang
	jsr	OSASCI			; Output service flag

	lda	InfoByte		; Get saved info byte
	and	#LanguageRom		; Test Language rom flag
	beq	DoRomsNotLang
	
	lda	#'L'			; Show that this is a Language
	jmp	DoRomsFlagsEnd
	
.DoRomsNotLang
	lda	#Space			; show that it is not

.DoRomsFlagsEnd
	jsr	OSASCI			; Output service flag
	
	ldx	#RomParam2Mess-TextMessBase	; print message
	jsr	DisplayMess
	
	lda	InfoByte		; Get saved info byte
	jsr	OutHexByte		; Output info byte as hex
	
	ldx	#RomParam3Mess-TextMessBase	; print message
	jsr	DisplayMess
	
        lda     ListFlags               ; Clear discovery flags
        and     #FlagVerbose            ; Mask out flags
        sta     ListFlags
        
	ldy	#DoRomsRamCodeEnd-DoRomsRamCode	; Get length to copy
{
.StackCopyLoop
	lda	DoRomsRamCode,Y		; Get a byte
	pha				; Put on stack
	dey				; Decrement pointer
	bmi	StackCopyDone
	jmp	StackCopyLoop		; If more do them
	
.StackCopyDone
	tsx				; Get exec address	
	lda	#&01			; Stack is in page &01
	pha
	
	txa				; Now stack SP
	pha				
}
	rts				; Call stacked code
	
.DoRomsNextEOL
	tsx				; clean up stack
	txa
	sec
	adc	#DoRomsRamCodeEnd-DoRomsRamCode
	tax
	txs
	
	jsr	OutEOL
.DoRomsNext
	lda	SourceID		; Check if we have finished
	cmp	DestID			; finished when source=dest
	beq	DoRomsEnd		; Yep, exit
	
	dec	SourceID		; Do next rom
	jmp	DoRomsLoop		; No : keep going
	
.DoRomsEnd
	jmp	CommandEndHandled

;
; This code is copied to RAM (on the stack), as it has to execute 
; from outside of the banked rom, as it will change banks !
;
	
.DoRomsRamCode

if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	SourceID	; Get ID to process
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	SourceID	; Get ID to process
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif

	ldx	#SWRTitle	; point to title
.DoRomsTitleLoop
	lda	SWBase,x	; get a byte from title
	beq	DoRomsVer	; Zero=end of title, look for version
	
        cmp     #&20            ; If it's a control char ignore it
        bcc     DoNextTitleChar
        
        cmp     #&7F            ; is it ASCII ?
        bcs     DoRomsVer	; no:end of title, look for version
        
	jsr	OSASCI		; Output it
.DoNextTitleChar
	inx			; increment pointer
	cpx	#$FF		; End of page ?
	beq	DoRomsRamAbort	; yes : abort
	bne	DoRomsTitleLoop	; no : do next	
	
.DoRomsVer
	inx			; increment past zero
	lda	#Space		; print a space
	jsr	OSASCI
	
.DoRomsVerLoop
	lda	SWBase,x	; get a byte from title
	beq	DoRomsRamAbort	; Zero=end of versio, we're done
	cmp	#CR		; quit if CR or LF in string
	beq	DoRomsRamAbort	; Zero=end of versio, we're done
	cmp	#LF		; quit if CR or LF in string
	beq	DoRomsRamAbort	; Zero=end of versio, we're done
	cmp     #&7F            ; is it ASCII ?
        bcs     DoRomsVerNext   ; nope skip
        cmp     #&20            ; If it's a control char ignore it
        bcc     DoRomsVerNext   ; yep skip
        
	jsr	OSASCI		; Output it
.DoRomsVerNext
	inx			; increment pointer
	cpx	#$FF		; End of page ?
	beq	DoRomsRamAbort	; yes : abort
	bne	DoRomsVerLoop	; no : do next	

.DoRomsDiscover
        

.DoRomsRamAbort

if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	OurID	        ; Retieve our ROM id
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom
else
	lda	OurID		; Retieve our ROM id
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif

	jmp	DoRomsNextEOL		; Back to ROM
.DoRomsRamCodeEnd
