;*************************************************************************
; *SRSave <FileName> <RomAddr> <EndAddr> <id> (<Q|S>)
; *SRSave <FileName> <RomAddr> <+Length> <id> (<Q|S>)                                         
;
; Saves a file into sideways ram id at specified address.
;
; Q=Quick, use OSHWM to (HIMEM-1) as a buffer
; S=Screenmem, use HIMEM to &7FFF as a buffer
; 
; If neither Q or S is specified, then the &200 byte cassette buffer is used.
;*************************************************************************
;
; Intemediate jumps, so they can be in range of Bxx instructions.
;

.DoSaveParamError
	jmp	ErrMsgParamError
	
.DoSaveSramAddrError
	jmp	ErrMsgSramAddrError

.DoSaveBadSRId
	jmp	ErrMsgBadSRId


.DoSRSave
	jsr	NextNonSpace		; Skip past spaces

	ldx	StackFrame		; Point to stack frame
	tya
	sta	SaveCliPtr,x		; Save ptr to name for later use
	
	jsr	NextSpace		; skip past filename
	bcs	DoSaveParamError	; Error : exit
	
	jsr	TerminateFilename	; Terminate filename for OSFILE
	jsr	NextNonSpace		; Skip past spaces

; Get Save address into A:X
	jsr	GetHexWord		; Get hex word
	bcs	DoSaveSramAddrError	; Error : exit

; Check Save address is in the $8000-$BFFF range
	
	cmp	#SWPage		
	bcc	DoSaveSramAddrError	; Error : exit, to low
	cmp	#OSRomPage
	bcs	DoSaveSramAddrError	; Error : exit
	
	sta	MemPtr+1		; Save in memory pointer
	stx	MemPtr

	jsr	NextNonSpace		; Skip past spaces
	
	jsr	GetHexWord		; Get hex word
	bcs	DoSaveSramAddrError	; Error : exit

	jsr	CalcSWREndLen		; Calculate offset and length
	bcs	DoSaveSramAddrError	; Error exit
	
; Get rom id
	jsr	GetDestID		; Get from command line
	bcs	DoSaveBadSRId		; Error : exit
	
; If we make it here we have a valid set of required params.
; check for optional flags
.DoSaveNextFlag
	jsr	GetFlag			; Get next flag
	bcs	DoSaveFlagsDone
	
	jsr	DoDecodeFlag		; Decode flags
	jmp	DoSaveNextFlag		; Check for more

.DoSaveFlagsDone

	jsr	DoSetupFileBuf		; Setup fileBuf depending on flags

	lda	MemPtr,x		; Copy start address for later use
	sta	SaveStartAddr,x
	lda	MemPtr+1,x		
	sta	SaveStartAddr+1,x

;
; As OSGBPB has much lower performance than OSFILE for writing files
; we will check to see if we can write the whole file in one go, and
; use OSFILE to do this if we can. This requires that we get the size of 
; the file to be saved, and if it is smaller than or equal to the FileBuf
; then we will use OSFILE to save it in one go, else we will fall back to
; using the slower OSGBPB. 
;

	lda	DataLen+1,x		; Get page count we need to save
	cmp	DTAPages,x		; compare to size of buffer
	bcc	SRSaveUseOSF		; smaller use os file

	bne	SRSaveUseOSGP		; Not same (bigger), use OSGBPB

; if we reach here then the hi byte is the same, so if the length is 0 then we
; have an exact number of pages and can still use OSFILE
	
	lda	DataLen,x		; get low byte
	bne	SRSaveUseOSGP		; not zero, use OSGBPB
	
.SRSaveUseOSF	
	lda	#FlagTrue		; Flag that we are using OSFILE
	sta	OSFFlag,x
	jmp	SRSaveDoStack		; Go get the data from swr


.SRSaveNotFound
	jmp	ErrMsgFileNotFound

; Try opening the file
	
.SRSaveUseOSGP
	lda	#FlagFalse		; Flag that we are not using OSFILE
	sta	OSFFlag,x

	clc
	ldx	StackFrame		; Point to stack frame
;	lda	CLIPtr			; Get low byte of CLI pointer
;	adc	SaveCliPtr,x		; Add saved filename pointer
	lda	SaveCliPtr,x		; Get saved filename pointer
	tax				; move it to X
	ldy	CLIPtr+1		; Get high byte
	lda	#OSFDOpenWrite		; Open the file for write
	jsr	OSFIND			; Try opening the file
	
	cmp	#&00			; Check for valid handle 
	beq	SRSaveNotFound		; Not found, error
	
	ldx	StackFrame		; Point to Stack frame
	sta	GPHandle,x		; save file handle
;
; Push the copy code onto the stack, and execute it from there
;
.SRSaveNextBlock
	
.SRSaveDoStack
	ldy	#SRSaveRAMCopyEnd-SRSaveRAMCopy
.SRSaveDoStackLoop
	lda	SRSaveRAMCopy,y		; Get a byte
	pha				; save on stack
	dey				; decrement count
	bmi	SRSaveCallOnStack	; Done all ? yes, exit
	jmp	SRSaveDoStackLoop

.SRSaveCallOnStack
	lda	#0			; Fill in FileBufPtr LSB, alwsys 0 as on page 
	sta	FileBufPtr
	lda	DTABasePage,x		; Fill in FileBufPtr MSB
	sta	FileBufPtr+1

	tsx				; Get address to call
	lda	#StackPage
	pha
	txa
	pha
	rts				; call code on stack

.SRSaveOSFILE
	jmp	DoSRSaveOSFILE

.SRSaveCopyEnd	
	tsx				; clean up stack
	txa
	sec
	adc 	#SRSaveRAMCopyEnd-SRSaveRAMCopy
	tax
	txs
	
; Check to see if we should use OSFILE to save the file, and do so if needed

	ldx	StackFrame		; Point to our stack frame
	lda	OSFFlag,x		; Get OSFILE flag
	cmp	#FlagTrue		; set ?
	beq	SRSaveOSFILE		; yes go do it

; Fill in OSGBPB parameter block
	
.SRSaveFillParam
	
	lda	#0			; Offset always 0 as on page boundry
	sta	GPDataPtr,x
	
	lda	DTABasePage,x		; Base page of DTA
	sta	GPDataPtr+1,x
	
	lda	#&FF
	sta	GPDataPtr+2,x
	sta	GPDataPtr+3,x
	
	lda	DataLen+1,x		; Get current data length
	cmp	DTAPages,x		; Bigger than file buffer ?
	bcc	SRSaveSetLen		; no, set length
	
; if we have more total bytes to write than the size of FileBuf, then we write
; FileBuf bytes
	
	lda	#0			; Fill in bytes to write, LSB always zero as on page
	sta	GPLength,x
		
	lda	DTAPages,x		; Get no of pages
	sta	GPLength+1,x
	
	jmp	SRSaveSetTopLen		; set top bytes
	
.SRSaveSetLen
	lda	DataLen,x		; Fill in a size less than filebuf
	sta	GPLength,x
	
	lda	DataLen+1,x
	sta	GPLength+1,x
	
.SRSaveSetTopLen	
	lda	#0			; Top 2 bytes 0
	sta	GPLength+2,x
	sta	GPLength+3,x
;
; Now try saving the block to file.
;
.SRSaveWrite
	lda	#OSGPPutIgnore		; Put bytes, ignore pointer from block
	ldy	#StackPage		; Y:X->GBPBbuffer address
	ldx	CtrlBufAddr	
	jsr	OSGBPB	
	
	bcc	SRSaveCalcBytes		; No error, continue
	
	lda	#ErrIO			; Flag error
	jmp	SRSaveCloseFile 	; Error, clean up and exit
	
;
; Calculate bytes left by subtracting the FileBuf size from the DataLen.
;
.SRSaveCalcBytes
	sec
	ldx	StackFrame		; Point at stack frame
	lda	DataLen,x		; Do LSB
	sbc	#0			; Zero as always page sized
	sta	DataLen,x
	
	lda	DataLen+1,x		; Now do MSB
	sbc	DTAPages,x
	sta	DataLen+1,x
	
	bmi	SRSaveDone		; When MSB is minus then we have processed all bytes
	
	jmp	SRSaveNextBlock		; else process the next block

.SRSaveDone
	lda	#ErrOK			; Flag no error
	
.SRSaveCloseFile	
	pha
	ldx	StackFrame		; Point to GBPBBuff
	ldy	GPHandle,x		; Get the file handle
	lda	#OSFDCloseFile		; Close the file
	jsr	OSFIND
	pla

	cmp	#ErrOK
	bne	SRSaveError		; Error, display message
	
	jmp	CommandEndHandled

.SRSaveError
	ldx	#IOErrorMess-TextMessBase
	jsr	DisplayMess
	jmp	CommandEndHandled

;
; Save data in FileBuf using the much faster OSFILE to do it in one chunk.
;

.DoSRSaveOSFILE
	ldx	StackFrame		; Point to stack frame
;	lda	CLIPtr			; Get low byte of CLI pointer
;	adc	SaveCliPtr,x		; Add saved filename pointer
	lda	SaveCliPtr,x		; Get saved filename pointer
	sta	SFOSFFNameAddr,x
	lda	CLIPtr+1		; Get CLI page
	sta	SFOSFFNameAddr+1,x	; Save in control block

; Fill in load and exec addresses, equal to first address from SWR
	lda	SaveStartAddr,x
	sta	SFOSFLoadAddr,x
	sta	SFOSFExecAddr,x

	lda	SaveStartAddr+1,x
	sta	SFOSFLoadAddr+1,x
	sta	SFOSFExecAddr+1,x
	
	lda	#&FF			; Flag in IOproc memory
	sta	SFOSFLoadAddr+2,x
	sta	SFOSFExecAddr+2,x
	sta	SFOSFLoadAddr+3,x
	sta	SFOSFExecAddr+3,x
	
; Fill in start address for save
	lda	#0			; zero as on page boundry
	sta	SFOSFStartAddr,x
	lda	DTABasePage,x		; get page no
	sta	SFOSFStartAddr+1,x
	lda	#&FF			; Flag in IOproc memory
	sta	SFOSFStartAddr+2,x
	sta	SFOSFStartAddr+3,x

; Add start addr to data len to work out end addr
	clc
	lda	#0			; zero as on page boundry
	adc	DataLen,x
	sta	SFOSFEndAddr,x
	
	lda	DTABasePage,x		; now do page count
	adc	DataLen+1,x
	sta	SFOSFEndAddr+1,x
	
	lda	#&FF			; Flag in IOproc memory
	sta	SFOSFEndAddr+2,x
	sta	SFOSFEndAddr+3,x
	
	lda	#OSFSaveBlock		; Save a block of memory
	ldy	#StackPage		; Y:X-> address of OSFBuf
	ldx	CtrlBufAddr
	jsr	OSFILE
	
	jmp	CommandEndHandled

	
;
; Code copied into RAM on the machine stack to copy bytes from
; the SWR to the file buffer.
; Always copies a full buffer full even if this overflows, as 
; the write to file code will only save up to the last byte.
;
	
.SRSaveRAMCopy
if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	DestID	        ; Get ID to process
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	DestID		; Select dest page
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif	

	ldy	#0		; Zero byte count
	ldx	StackFrame	; Point to stack frame
	
.SRSaveRAMCopyLoop

	lda	(MemPtr),y	; get a byte from SWR 
	sta	(FileBufPtr),y	; put it in buffer
	iny			; increment SWR pointer
	cpy	#0		; done a whole page ?7
	bne	SRSaveRAMCopyLoop	; nope do next byte

	inc	MemPtr+1	; Move to next SWR page
	inc	FileBufPtr+1	; Move to next file buff page

	sec
	lda	FileBufPtr+1	; Get MSB of file buff pointer
	sbc	DTABasePage,x	; Subtract base page
	cmp	DTAPages,x	; done a buffer full yet ?
	bne	SRSaveRAMCopyLoop	; nope keep getting bytes
	
if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	OurID	        ; Retieve our ROM id
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	OurID		; Retieve our ROM id
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif
	jmp	SRSaveCopyEnd
.SRSaveRAMCopyEnd
