;*************************************************************************
; *SRZAP <idfirst> (<idlast>)
;
; Zero out the sideways memory banks staring at idfirst and ending at 
; idlast. If only idfirst is specified, then only that single bank is
; zeroed
;*************************************************************************

.DoZapParamError
	jmp	ErrMsgParamError

.DoSRZAP
	jsr	Get1or2RomIDS	; Get rom IDs
	bcs	DoZapParamError	; Out of range, flag error
	jsr	SwapIDS		; swap IDs if needed

;	
; If we get to here then we have valid start and end parameters
; attempt to zap the memory.
;	
;
; Copy the RAM zeroing code onto the stack !
;
	
	ldy	#ZeroCodeEnd-ZeroCode	; Get length to copy
.ZeroStackCopyLoop
	lda	ZeroCode,Y		; Get a byte
	pha				; Put on stack
	dey				; Decrement pointer
	bmi	ZeroStackCopyDone
	jmp	ZeroStackCopyLoop	; If more do them
	
.ZeroStackCopyDone
	tsx				; Get exec address	
	lda	#&01			; Stack is in page &01
	pha
	
	txa				; Now stack SP
	pha				
	
.ZeroLoop
	lda	SourceID		; Get ROM ID
	cmp	OurID			; prevent us overwriting ourselves 
	beq	NoZap

	pha				; Save rom ID
	jsr	GetRomTypeTable		; Get pointer to rom type table in Y:X
	
	stx	MemPtr			; save in MemPtr
	sty	MemPtr+1
	pla				; recover RomID
	tay				; Zero zapped rom's byte
	lda	#&00
	sta	(MemPtr),Y
	
	lda	#&00			; Save start address for zap
	sta	MemPtr
	lda	#SWPage
	sta	MemPtr+1

	rts				; Call code on stack

.NoZap
	pla				; Drop call address
	pla	
	
.ZapEndRamExec

	lda	SourceID		; Have we done all ?
	cmp	DestID
	beq	ZapAllDone		; Yep : exit
	
	inc	SourceID		; increment source ID
	jmp	ZeroStackCopyDone	; Do next ram
	

.ZapAllDone
	tsx				; clean up stack
	txa
	sec
	adc 	#ZeroCodeEnd-ZeroCode
	tax
	txs
	
	jmp	CommandEndHandled
	
;
; This code is copied to RAM (on the stack), as it has to execute 
; from outside of the banked rom, as it will change banks !
;

.ZeroCode
if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	SourceID	; Get ID to process
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	SourceID	; Select source page
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif	
	
	ldy	#&00
.ZeroLoop1
	lda	#&00		; Zero a
.ZeroLoop2	
	sta	(MemPtr),Y	; Put it in SWram
	iny			; Increment mem pointer low byte
	bne	ZeroLoop2	; Continue if not Zero
	
	inc	MemPtr+1	; Increment high byte
	lda	MemPtr+1	; Get memptr high byte
	cmp	#OSRomPage	; End of SRAM area ?
	bne	ZeroLoop1
	
if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	OurID	        ; Retieve our ROM id
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	OurID		; Retieve our ROM id
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif

	jmp	ZapEndRamExec	; Return to caller
.ZeroCodeEnd
