;
; Scan for commands
;

;
; Enter from ROM service call ROMCmdNotKnow/$04
;
; On entry, A=$04, X=rom number, 
; ($f8),Y=address of first non space after *
;

.DoCommand
; Save registers
	pha			; Push a
	txa	
	pha			; Push x
	tya	
	pha			; Push Y

; Setup command stack frame
	tsx			
	txa
	sec

.SetStack	
	sbc	#StackFrameLen	; Make room for stack frame
	sta	StackFrame	; Save address
	
	sbc	#1		; move stack beyond our frame
	
	tax
	txs			; put back in sp
	
	clc
	lda	StackFrame	; Calculate OSGBPBBufAddr
	adc	#GBPBBuf-StackBase
	sta	CtrlBufAddr

	ldx	StackFrame	; Make sure flags are always 0
	lda	#0
	sta	Flags,x

	lda	RomSel		; Get rom ID from OS
	sta	OurID		; save in our ID
	
	ldx	#&00		; init at beginning of command block

.ResetY	
;	dey			; Dec Y as loop increments it
.ScanNextCommand
	tya
	pha			; Save Y again so we can get it back later
	
	lda	CommandTable,X	; Check for end of table
	cmp	#&00	
	beq	NoCommandFound	; Yes: we didn't find anything, exit
	dex			; Decrement as loop increments it
	
	lda	(CLIPtr),Y	; Get a byte from CLI
	AND	#&DF		; Force to upper case
	cmp	#PrefixChar	; equal to prefix char ?
	beq	ScanCommandLoop	; yes, skip over it
	dey			; no decrement y, to compensate for increment below
	
.ScanCommandLoop
	iny			; increment pointers
	inx			
	
	lda	(CLIPtr),Y	; Get a byte from CLI
	AND	#&DF		; Force to upper case
	cmp	CommandTable,X	; Compare against our table
	beq	ScanCommandLoop	; Matches ok, so keep going

	lda	CommandTable,X	; Get last byte compared
	cmp	#&FF		; End of command
	beq	ExecCommand	; Yes : go off and exec command

.FindEndOfCommand
	inx			; increment x
	lda	CommandTable,X	; Get last byte compared
	cmp	#&FF		; End of command
	bne	FindEndOfCommand 

; We have found the end of the command, skip over the call address
	inx
	inx
	inx
	
	pla			; Restore CLI pointer
	tay			; back to Y
	jmp	ResetY			

;	jmp	ServiceEndNotHandled

.NoCommandFound
	pla			; Drop saved Y
	jmp	CommandEndNotHandled
	
.ExecCommand
	pla			; Drop saved Y
	inx			; Point at exec address
	inx
	lda	CommandTable,X	; Retrieve EXEC address and put on stack
	pha
	dex
	lda	CommandTable,X	; 
	pha
	
	rts			; jump to it !
	
;
; Clean up stack, all commands should call this to terminate.
;

.CommandEndHandled
	clc
	lda	StackFrame	; Get stack frame pointer
	adc	#StackFrameLen	; Move back past our frame
	tax			; put back in sp
	txs
	jmp	ServiceEndHandled	

.CommandEndNotHandled
	clc
	lda	StackFrame	; Get stack frame pointer
	adc	#StackFrameLen	; Move back past our frame
	tax			; put back in sp
	txs
	jmp	ServiceEndNotHandled	
	
;
; Command table, format :-
;
; Offset	Length	Purpose
; 0..n 		n	CommandName1
; n+1		1	$FF Terminator
; n+2		2	Address of command handler
;
; Repeat above for each command, terminate with $00 in 
; first byte of command name. Entire command block must be
; less than 255 bytes total !
;
	
.CommandTable
	EQUS	"SRZAP",&FF
	EQUW	DoSRZAP-1
	
	EQUS	"SRCOPY",&FF
	EQUW	DoSRCOPY-1
	
	EQUS	"SRCMP",&FF
	EQUW	DoSRCMP-1
	
	EQUS	"SRLOAD",&FF
	EQUW	DoSRLoad-1
	
	EQUS	"SRSAVE",&FF
	EQUW	DoSRSave-1
	
	EQUS	"SRREAD",&FF
	EQUW	DoSRRead-1
	
	EQUS	"SRWRITE",&FF
	EQUW	DoSRWrite-1
	
;	EQUS	"SRPOKE",&FF
;	EQUW	DoSRPoke-1
;	
;	EQUS	"SRPEEK",&FF
;	EQUW	DoSRPeek-1
	
	EQUS	"NOP",&FF
	EQUW	DoNop-1
	
	EQUS	"ROMS",&FF
	EQUW	DoRoms-1
	
	EQUS	"RESET",&FF
	EQUW	DoReset-1
	
	
; end of table
	EQUB	&00

;
; Find next non-space character at (CLIPtr),Y
; exits with A=first non space.
;

.NextNonSpace
	lda	(CLIPtr),Y	; get a character from CLI
	cmp	#LF		; End of line ?
	beq	FoundEOL	; yes : exit, error

	cmp	#Space		; Space ?
	bne	NonSpace	; nope : exit
	iny			; Yes : increment and check next
	jmp	NextNonSpace

.NonSpace
	clc
	rts

;
; Find next space character at (CLIPtr),Y
; exits with A=first space, and carry clear
; or end of command line and carry set.
;
.NextSpace
	lda	(CLIPtr),Y	; get a character from CLI
	cmp	#Space		; Space ?
	beq	FoundSpace	; yes : exit, ok
	
	cmp	#LF		; End of line ?
	beq	FoundEOL	; yes : exit, error
	
	iny
	jmp	NextSpace	; Check next
	
.FoundSpace
	clc			; Flag ok
	rts
.FoundEOL
	sec			; Flag error
	rts
	
;
; Get next flag, first finds next space, then next non space.
; flag returned in a and carry clear on success.
;

.GetFlag
	jsr	NextSpace	; Skip past any non-space chars
	bcs	GetFlagDone	; error : exit
.GetFlag2	
	jsr	NextNonSpace	; Find next flag
	
.GetFlagDone
	rts

;
; Call to fill in the terminator at the end of a filename, as OSFILE 
; needs it.
;
.TerminateFilename
	pha			; save a
	lda	#LF		; Terminator 
	sta	(CLIPtr),Y	; put at end of filename
	iny			; increment past terminator
	pla			; restore a
	rts

;
; Convert hex char in A to binary, and return it in A
; Carry set on error, clear on success.
;

.ConvHexDigit
	cld	
	cmp	#&41		; Is the charcater < "A"
	bcc	BelowA
	
	and	#&DF		; force to upper case
	sec
 	sbc	#&37		; Convert A..F
	cmp	#&10		; greater than 15 ?
	bcs	HexBinError	; yes : error

.HexBinOK
	and	#&0F		; make sure only LSN set
	clc			; Flag ok
	rts

.BelowA
	sec
	sbc	#&30		; Convert 0..9
	cmp	#&0A		; Greater than 9 ?
	bcc	HexBinOK	; Nope, result ok
	
.HexBinError
	sec			; flag error
	rts

;
; Get a hex digit from the command line, returns byte in A
; On exit Y points to the first invalid character, so it is
; only advanced if a valid hex digit is found
; Carry set on error, clear on success.
; 

.GetHexDigit
	lda	(CLIPtr),Y	; get a character from CLI
	jsr	ConvHexDigit	; Convert to binary in A
	bcs	GetHexError	; error : exit
	iny			; increment pointer
	clc			; flag no error
	rts
	
;
; Get a hex byte from the command line, returns byte in A
; On exit Y points to the first unprocessed character
; Carry set on error, clear on success.
;
	
.GetHexByte
	jsr	GetHexDigit	; Get MSB
	bcs	GetHexError	; error : exit
	
	pha			; save on stack
	
	jsr	GetHexDigit	; Any more hex chars ?
	bcs	GetHexLSNOnly	; nope return LSN only

	and	#&0F		; Mask out top bits
	pha			; save LSN on stack
	
	tsx			; get pointer to LSN/MSN on stack
	lda	StackBase+2,x	; recover MSN
	asla			; Move to MS nibble
	asla
	asla
	asla	
	and	#&F0		; Mask out bottom bits
	ora	StackBase+1,x	; merge with LSN byte on stack

	inx			; clean up stack
	inx
	txs
	
	clc			; flag no error
	
.GetHexError
	rts
	

.GetHexLSNOnly
	pla			; drop saved a
	clc			; flag no error
	rts

;
; Get hex word, get a hex word from the command line and 
; return it in A:X, carry will be set on error.
;
; The hex word will be built up on the stack, and then
; popped off at the end, locations on the stack will be :-
;
; SP+1	Most recently read hex digit
; SP+2  LSB
; SP+3	MSB
;

.GetHexWord
	lda	(CLIPtr),Y	; Get next character
	cmp	#OffsetFlagChar	; Check for offser marker
	bne	GetHexWordAbs	; nope don't skip it
	iny			; else skip past +
	jmp	GetHexWordGet
	
.GetHexWordAbs
	lda	#0		; Set absolute mode
	
.GetHexWordGet
	ldx	StackFrame	; Point at stack frame
	sta	OffsetFlag,x	; save flag

; Initially zero counters on stack

	lda	#0		; Zero word to start with
	pha			; MSB
	pha			; LSB
	pha			; Hex digit

	tsx			; get pointer to stack
	
	jsr	GetHexDigit	; Get first digit
	bcs	GetHexWordError	; error : exit
	sta	StackBase+1,x	; Save digit
	jsr	MergeDigit	; Merge it
	
	jsr	GetHexDigit	; Get next digit
	bcs	GetHexWordDone	; No more digits, exit
	sta	StackBase+1,x	; Save digit
	jsr	MergeDigit	; Merge it

	jsr	GetHexDigit	; Get next digit
	bcs	GetHexWordDone	; No more digits, exit
	sta	StackBase+1,x	; Save digit
	jsr	MergeDigit	; Merge it

	jsr	GetHexDigit	; Get next digit
	bcs	GetHexWordDone	; No more digits, exit
	sta	StackBase+1,x	; Save digit
	jsr	MergeDigit	; Merge it

.GetHexWordDone
	pla			; remove last digit
	pla			; Get LSB
	tax
	pla			; and MSB
	clc			; flag no error
	rts
	
.GetHexWordError
	pla			; Clean stack
	pla
	pla
	sec			; flag error
	rts
;
; Merge read hex byte with word on stack, on entry the 
; stack should have 3 bytes indexed by x on as follows :-
;
; X+1	Most recently read hex digit to be merged
; X+2  	LSB
; X+3	MSB
;

.MergeDigit
	lda	StackBase+3,x	; get MSB

	asla			; Move to shift it left
	asla
	asla
	asla	
	and	#&F0		; mask out ls nibble
	sta	StackBase+3,x	; save it back
	
	lda	StackBase+2,x	; get LSB
	lsra			; move MSN of LSB to LSN
	lsra
	lsra
	lsra
	and	#&0F		
	ora	StackBase+3,x	; Merge with MSB
	sta	StackBase+3,x	; save it back
	
	lda	StackBase+2,x	; Now move LSN of LSB to MSN
	asla			; Move to shift it left
	asla
	asla
	asla	
	and	#&F0		; Mask out bottom 4 bits
	ora	StackBase+1,x	; Merge in most recent digit
	sta	StackBase+2,x	; save it back
	
	rts

	
;
; Calculate length and end address of a block of SWR
; either a valid sram offset, or a valid absolute swr address
; valid ranges are :- 
; &0000 to &3FFF for offsets, as SWR is 16K
; &8000 to &BFFF for absolute as this is the valid range of the 
; sram.
;
; On entry :-
;       MemPtr needs to be setup with a valid start address.
;       A:X should contain the end addr or offset.
;
; On exit EndAddr and DataLen will be setup correctly, or carry 
; set on error
;
	
.CalcSWREndLen
	jsr	GetOffsetInCarry	; Get offset/absolute flag
	bcs	ValidSWROffset		; yep flag it & return

	cmp	#SWPage		
	bcc	CalcSWRInvalid		; Error : exit, to low
	cmp	#OSRomPage
        bcs	CalcSWRInvalid		; Error : exit


.CalcSWR
	jsr	CalcLengthFromEnd	; Calculate length of block from end

	cmp	#OSRomPage		; check for start+len > &C000
	bcs	CalcSWRInvalid		; Yep : invalid
	
	clc				; flag no error
	rts

.ValidSWROffset
	jsr	CalcEndFromLen		; Calculate end addr from length
	clc				; Flag no error
	rts
	
.CalcSWRInvalid
	sec				; Flag error
	rts

.CalcLengthFromEnd
	pha				; Setup to access stack frame
	txa
	pha
	ldx	StackFrame

	pla				; retrieve saved x
	sta	EndAddr,x		; Save end address
	pla				; retrieve saved a
	sta	EndAddr+1,x

; now calculate the length
	sec				
	lda	EndAddr,x		; Do MSB
	sbc	MemPtr
	sta	DataLen,x
	
	lda	EndAddr+1,x		; Do MSB including carry
	sbc	MemPtr+1
	sta	DataLen+1,x
	rts
	
.CalcEndFromLen
	pha				; Setup to access stack frame
	txa
	pha
	ldx	StackFrame

; The specified data len is stored in DataLen, a second copy is stored in
; DataLenTmp, and decremented by 1 so that both are valid.

	pla				; retrieve saved x
	sta	DataLen,x		; save in length
	sta	DataLenTmp,x
	pla				; retrieve saved a
	sta	DataLen+1,x	
	sta	DataLenTmp+1,x
	
; Since we are calculating the LAST byte processed, we need
; to decrement the length by one or elese we will point to the next
; byte to be processed.

	dec	DataLenTmp,x		; decrement LSB
	lda	DataLenTmp,x		; check for underflow
	cmp	#&FF			
	bne	CalcEndFromLenDoCalc	; no, do calculation
	dec	DataLenTmp+1,x		; decrement MSB

.CalcEndFromLenDoCalc
; now calculate the end address
	clc
	lda	MemPtr			; Do LSB
	adc	DataLenTmp,x
	sta	EndAddr,x
	
	lda	MemPtr+1		; Do MSB including carry
	adc	DataLenTmp+1,x
	sta	EndAddr+1,x
	rts
	
;
; Get offset flag in carry
;
	
.GetOffsetInCarry
	pha				; save regs
	txa
	pha

	ldx	StackFrame		; Point to stack frame
	lda	OffsetFlag,x		; Get ascii offset flag
	cmp	#OffsetFlagChar		; Offset ?
	bne	GetOffsetAbs		; nope flag absolute

	pla				; restore regs
	tax
	pla
	sec				; Flag absolute
	rts
	
.GetOffsetAbs	
	pla				; restore regs
	tax
	pla
	clc				; Flag offset
	rts
	
;
; Error message handlers
;

.ErrMsgSramAddrError
	ldx	#SramAddrErrorMess-TextMessBase
	jmp	MessageExit

.ErrMsgBadSRId
	ldx	#BadSRIdMess-TextMessBase
	jmp	MessageExit

.ErrMsgFileNotFound
	ldx	#FileNotFoundMess-TextMessBase
	jmp	MessageExit

.ErrMsgFileTooBig
	ldx	#FiletooBigMess-TextMessBase
	jmp	MessageExit

	
.ErrMsgParamError
	ldx	#ParamErrorMess-TextMessBase
.MessageExit
	jsr	DisplayMess
	jmp	CommandEndHandled


;
; Get two rom IDs from the command line and save them in SoureID and
; DestID, exits with carry set on error, clear otherwise
; Both IDs must be supplied to succeed.
;

.GetRomIDS
	jsr	MakeIDSInvalid	; Make both IDS invalid
	jsr	GetSourceID	; Get source rom id
	bcs	GetRomIDSErr	; Error flagged exit
	jsr	GetDestID	; Get dest rom id
	bcs	GetRomIDSErr	; Error flagged exit

.GetRomIDSExit
	clc
	rts
	
.GetRomIDSErr
	sec
	rts
	

.MakeIDSInvalid
	lda	#InvalidID	; Assume both invalid
	sta	SourceID
	sta	DestID
	rts
	
;
; Get 1 or 2 rom ids from command line and save to SourceID and DestID.
; If only one ID supplied then source ID is copied to DestID.
;
	
.Get1or2RomIDS
	jsr	MakeIDSInvalid	; Make both IDS invalid
	jsr	GetSourceID	; Get source rom id
	bcs	GetRomIDSErr	; Error flagged exit
	jsr	GetDestID	; Get dest rom id
	bcc	GetRomIDSExit	; no error, both specified
	
	lda	SourceID	; Error, so copy source to dest
	sta	DestID
	clc			; Flag no error
	rts

;
; Get source ROM ID
;	
	
.GetSourceID
	jsr	GetID		; get source id
	bcs	GetSrcErr	; error don't save
	sta	SourceID	; Save source rom ID
.GetSrcErr
;	iny			; increment cmd line pointer
	rts

;
; Get destination rom ID
;

.GetDestID
	jsr	GetID		; get source id
	bcs	GetDestErr	; error don't save
	sta	DestID		; Save dest rom ID
.GetDestErr
;	iny			; increment cmd line pointer
	rts
	
;
; Get rom ID from command line into A, carry set on error
;
	
.GetID
	jsr	NextNonSpace	; Find next non-space character
	cmp	#LF		; Any params ?
	
	beq	GetIDError	; no : param error

; Get first rom ID
	jsr	GetHexDigit	; Convert char to binary, returns cc if ok
	rts
	
.GetIDError
	sec			; Flag error
	rts

;
; Swap first and last if needed so that sourceid is the lower. 
;

.SwapIDS
	lda	SourceID	; get source ID
	cmp	#InvalidID	; valid ID ?
	beq	NoSwapID	; nope : exit
	cmp	DestID		; compare to dest
	bcs	NoSwapID	; source is lower don't swap
	
	pha			; save SourceID on stack
	lda	DestID		; get dest
	sta	SourceID	; save in source
	pla			; retrieve saved Source
	sta	DestID		; and put in dest
	
.NoSwapID
	rts
	

;
; Get rom type table pointer in Y:X
;

.GetRomTypeTable
	ldx	#$00			; load these values to prevent changing pointer !
	ldy	#$FF
	lda	#OSBReadRomInfL		; Read rom type table
	jsr	OSBYTE
	rts

	include "CmdZap.asm"
	include "CmdCopy.asm"
	include "CmdCmp.asm"
	include "CmdLoad.asm"
	include "CmdSave.asm"
	include "CmdReadWrite.asm"
	include "CmdRoms.asm"
;	include	"CmdPeekPoke.asm"
	
.DoNop
	ldx	#TextNOP-TextMessBase
	jsr	DisplayMess
	jmp	CommandEndHandled

.DoReset
        LDA     #&7F            ; Reset system VIA        
        STA     SystemVIA+&0E	; &FE4E                   
        JMP     (ResetVect)    	; Reset machine             
	
