;
; GiveHelp.asm, Handle *HELP for this ROM
;

.DoHelp
; Save registers
	pha			; Push a
	txa	
	pha			; Push x
	tya	
	pha			; Push Y

; Check for return, if so *HELP with no parameters
	lda	(CLIPtr),Y		
	CMP	#LF
	bne	MoreHelp	; Nope, look in our table
	
	jmp	JustHelp
	
.MoreHelp
	ldx	#&FF		; Setup X so loop increments it to 0
	dey			; Decrement Y as loop increments it
	
.Compare
	iny			; Increment pointers
	inx
	
	lda	(CLIPtr),Y	; Get a character
	AND	#&DF		; Force to upper case
	cmp	Table,X		; Compare against our table
	beq	Compare		; Matches ok, so keep going

	lda	Table,X		; Get last byte compared
	cmp	#&FF		; End of table ?
	beq	GiveHelp	; Yep, so give the help
	jmp	ServiceEndNotHandled	; no : exit

.GiveHelp
	lda	#HelpFull		; Flag full title
	jsr	HelpRomTitle	; Display rom title
	lda	#LO(HelpTextCommands)	; Pointer to command help
	sta	MemPtr
	lda	#HI(HelpTextCommands)	; Pointer to command help
	sta	MemPtr+1
	
	jsr	HelpMess	; Display the rom name/version
	jmp	HelpEnd		; Done, exit

;
; Display help when just *HELP typed
;
	
.JustHelp
	lda	#HelpFull		; Flag full title
	
	jsr	HelpRomTitle	; Display rom title
	lda	#LO(HelpTextSections)
	sta	MemPtr
	lda	#HI(HelpTextSections)
	sta	MemPtr+1
	
	jsr	HelpMess	
	
	jmp	HelpEnd		; Done, exit

.HelpRomTitle
	pha			; save flag
	cmp	#HelpFull	; Full help ?
	bne	HelpNoLF
	
	lda	#LF		; display new line
	jsr	OSASCI
	
.HelpNoLF
	lda	#LO(HelpText)
	sta	MemPtr
	lda	#HI(HelpText)
	sta	MemPtr+1
	jsr	HelpMess	; Display the rom name/version
	
	pla			; retrieve flag
	beq	HelpNoDate	; Zero, don't display date
	
	inc	MemPtr		; Move past zero term
	jsr	HelpMess	; Display the rom name/version
	
.HelpNoDate

	lda	#LO(HelpRomNo)
	sta	MemPtr
	lda	#HI(HelpRomNo)
	sta	MemPtr+1
	jsr	HelpMess	; Display the rom name/version

	lda	RomSel		; Get current rom number
	and 	#&0F		; mask out rom number
	clc
	adc	#&30		; Convert to ascii
	cmp	#&3A		; >9, convert to letter
	bcc	ShowID
	adc	#&07		; Convert to A..F
	
.ShowID
	jsr	OSASCI		; print it

	inc	MemPtr		; Move past zero terminator
	jsr	HelpMess	; Display the rest of the title 		
	rts

;
; Display help message at MemPtr
;

.HelpMess
	ldy	#0
.HelpMessLoop	
	lda	(MemPtr),Y	; Get a byte
	cmp	#0		; End of string ?
	beq	HelpMessEnd	; Yep Exit
	
	jsr	OSASCI		; Write the character
	inc	MemPtr		; Increment pointer
	lda	MemPtr		
	bne	HelpMessLoop	; same page, keep going
	inc	MemPtr+1	; move to next page
	jmp	HelpMessLoop	; do next char
.HelpMessEnd
	rts	
	
;
; Display help message at TextMessBase+X
;

.DisplayMess
	lda	TextMessBase,X	; Get a byte
	cmp	#0		; End of string ?
	beq	DisplayMessEnd	; Yep Exit
	
	jsr	OSASCI		; Write the character
	inx			; Increment pointer
	jmp	DisplayMess	; do next char
.DisplayMessEnd
	rts	
	
; Flag not serviced so other roms may also respond to *help
.HelpEnd
	jmp	ServiceEndNotHandled	
	
	
.Table
	EQUS	"RAMUTILS",&FF	

