;
; RamUtils.asm
;
; 2008, 2009 P.Harvey-Smith.
;
; This program is dedicated to my cat Rascal, 1994-04-02 to 2008-12-27,
; who brought much joy and happyness for all of her 14 years.
;
; This software is released free for personal use, however use for profit is 
; expressly forbidden without permission. 
;
; 2010-03-23, P.Harvey-Smith.
;   Added compile time option to build for Electron.
;
; 2013-03-13, P.Harvey-Smith.
;   Hopefully fixed rom paging code on the Electron so that it works 
;   correctly.
;

; Set this to 1 to build for the Electron, 0 to build for the Beeb.
;
;        Electron = 1

;
; This controls how (and if) the ROM to RAM copy code is assembled for the
; Plus3RR board.
;
; It should be one of the following values :
;	0	No copy code built
;	1	Plus3RR Copy code built.
;
	CopyCode	= 1

INCLUDE "BeebDefs.asm"
INCLUDE "LocalDefs.asm"


	ORG	&8000

; Warn us if we overflow sideways ram space
	GUARD	&C000

.RomStart
	EQUB	0,0,0				; Language entry null
	
	jmp	Service				; Jump to service routine
	
; ROM type byte
	EQUB	ServiceRom+CPU6502		; Service rom containing 6502 code
	
	EQUB	(Copyright-RomStart) MOD 256	; Offset of copyright notice

;Verion number
	EQUB	0
;Title string
	EQUS	"RamUtils",0
;Version number
	EQUS	"V0.98",0
	
	
.Copyright
	EQUB	0
	EQUS	"(C)2008-2013 Ramoth Software"
	EQUB	0
	
.Service
	CMP	#ROMSpoolExClose ; close spool ?
	beq	DoSpoolJMP	; yes : do copy
	
	CMP	#ROMAutoBoot	; Rom initialisation ?
	beq	DoInitJMP	; Yes Init rom
	
	cmp	#ROMCmdNotKnow	; Unknown command ?
	beq	DoCommandJMP	; yes scan / execute
	
	cmp	#ROMHelpMe	; *Help Issued ?
	beq	DoHelpJMP	; Yes, issue help
	rts
	
.DoSpoolJMP
	JMP	DoSpool
.DoInitJMP
	JMP	DoInit
.DoCommandJMP
	JMP	DoCommand
.DoHelpJMP
	JMP	DoHelp
	
.ServiceEndNotHandled
;Restore saved regs
	pla			; Pull Y
	tay
	pla			; Pull X
	tax
	pla			; Pull A
	
	RTS

.ServiceEndHandled
;Restore saved regs
	pla			; Pull Y
	tay
	pla			; Pull X
	tax
	pla			; Pull A
	lda	#0		; Flag handled
	RTS
	
INCLUDE "Message.asm"
INCLUDE	"GiveHelp.asm"
INCLUDE "RomInit.asm"
INCLUDE "Command.asm"
INCLUDE "Utils.asm"

	SKIPTO	&C000
.RomEnd

if (Electron=1)
	SAVE "ramutile",RomStart,RomEnd
else
	SAVE "ramutil",RomStart,RomEnd
endif
