;
; RomInit.asm, initialise rom at reset time, mainly just prints title !
;

.DoInit
; Save registers
	pha			; Push a
	txa	
	pha			; Push x
	tya	
	pha			; Push Y
	
	lda	#HelpShort	; Flag short title
	jsr	HelpRomTitle	; Display signon.

	lda	#LF		; Move to next line
	jsr	OSASCI

; Always end not handled, so lower priority roms can init
	jmp	ServiceEndNotHandled
	
.DoSpool
	jsr	DoRomCopy	; Copy Rom to ram.
	rts
;
; The Plus3 RAMROM is an addon for the Plus 3 / AP3 which adds 4
; sideways banks internally. It consists of 64K of rom and 64K of 
; RAM. At startup the ROM is mapped into the plus banks when reading
; the RAM is *ALWAYS* mapped when writing, so we read the ROM and 
; copy it to the RAM bys simply reading every byte and then writing 
; it again. 
; Once we have done this we flip a switch in the hardware to switch
; to the RAM for both reading and writing.
;

FirstCopyId	= 4		; First SWR bank to copy
LastCopyId	= 7		; Last SWR bank to copy
	
TestLoc		= &BFFF		; Try writing this location 
TestPat1	= &55
TestPat2	= &AA	


;
; Set Plus3 copy code flags as needed.
;
if (CopyCode>0)
  if (Electron=1)
    Plus3 = 1;
  else
    Plus3 = 2;
  endif
else
  Plus3 = 0;
endif
	

.DoRomCopy
if (Plus3>0)

	jsr	DoRomCopyTest	; Check to see if already in RAM
	bcc	DoRomCopyExit	; in RAM nothing to do

	ldx	#$00		; Read cold / warm boot flag
	ldy	#$FF
	lda	#OSBRWLastBrkTyp
	jsr 	OSBYTE
	
	txa
	cmp	#ResetSoft	; Just break pressed ?
	beq	DoRomCopyNone	; Don't copy roms just go to ram mode

	lda	RomSel		; Get our ID
	sta	OurID

if (Plus3=1)
	lda	#FirstCopyId	; initialize
	sta	SourceID
	
	lda	#LastCopyId+1	
	sta	DestID
else
	lda	OurID		; get our id
	sta	SourceID	; save it
	
	clc	
	sbc	#12		; dest id will be 12 less, as in 4 rom steps
	sta	DestID
endif
	
;
; Copy the Copying code onto the stack !
;
	
	ldy	#RRCopyCodeEnd-RRCopyCode	; Get length to copy
.CopyStackCopyLoop
	lda	RRCopyCode,Y		; Get a byte
	pha				; Put on stack
	dey				; Decrement pointer
	bmi	CopyStackCopyDone
	jmp	CopyStackCopyLoop	; If more do them
	
.CopyStackCopyDone

.DoRomCopyNextOne
	lda	SourceID		; Get tCurrent rom ID
	cmp	DestID			; done all yet ?
	bcs	DoRomCopyEnd		; yep : done all

	tsx				; Get exec address	
	lda	#&01			; Stack is in page &01
	pha
	
	txa				; Now stack SP
	pha				

	lda	#&00			; Save start address for copy
	sta	MemPtr
	lda	#SWPage
	sta	MemPtr+1

	rts				; call copied code

;
; For the Electron & Plus3 our roms are are organised in a block of 4 from
; ID=4 to 7.
; On the BBC A/B because of the  way the rom sockets are arranged our ids
; are 4 apart so we have to decremtnt by 4 each time, as we copy the 
; highest rom number each time.
;

.DoRomCopyNext
if (Plus3=1)
	inc	SourceID		; move to next rom ID
else
	dec	SourceID		; move to next rom ID
	dec	SourceID		; move to next rom ID
	dec	SourceID		; move to next rom ID
	dec	SourceID		; move to next rom ID	
endif
	jmp	DoRomCopyNextOne	; Check and copy next one

.DoRomCopyEnd
	tsx				; clean up stack
	txa
	sec
	adc 	#RRCopyCodeEnd-RRCopyCode
	tax
	txs

.DoRomCopyNone	
	lda	#0			; Go to all RAM mode
if (Electron=1)
	sta	FloppyCtrlElk		; should not affect floppy
else
	sta	FloppyCTRL		; should not affect floppy
endif

.DoRomCopyExit
	rts

;
; Test to see if we are in ROM or RAM, try writing to TestLoc
; if this fails we are in ROM, if this successds we are in RAM.
;

.DoRomCopyTest
	lda	TestLoc		; save TestLoc
	tax
	
	lda	#TestPat1	; Write test pattern to TestLoc
	sta	TestLoc
	lda	TestLoc		; read it back
	cmp	#TestPat1	; Same ?
	bne	DoRomCopyIsROM

	lda	#TestPat2	; Write test pattern to TestLoc
	sta	TestLoc
	lda	TestLoc		; read it back
	cmp	#TestPat2	; Same ?
	bne	DoRomCopyIsROM

	txa			; Restore TestLoc
	sta	TestLoc

	clc			; Flag in RAM
	rts
	
.DoRomCopyIsROM
	sec			; Flag in ROM
	rts
	
.RRCopyCode	
if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	SourceID        ; Retieve ROM id to copy
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	SourceID	; Retieve ROM id to copy
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif

	ldy	#&00		
.CopyLoop
	lda	(MemPtr),Y	; get a byte from ROM
	sta	(MemPtr),Y	; write it to RAM
	iny			; Increment mem pointer low byte
	bne	CopyLoop	; Continue if not Zero
	
	inc	MemPtr+1	; Increment high byte
	lda	MemPtr+1	; Get memptr high byte
	cmp	#OSRomPage	; End of SRAM area ?
	bne	CopyLoop

if (Electron=1)
        lda     #PageOutBasicElk      ; Page out basic
        sta     RomSel
        sta     ROMPageElk

	lda	OurID	        ; Retieve our ROM id
	sta	RomSel		; Save here first
        sta     ROMPageElk      ; then page rom

else
	lda	OurID		; Retieve our ROM id
	sta	RomSel		; Save here first
	sta	ROMPage		; then page rom
endif

	jmp	DoRomCopyNext	; back to our rom
.RRCopyCodeEnd
else
	rts
endif